/*
 * ixjwin32.cxx
 *
 * QuickNet Internet Phone/Line JACK codec interface
 *
 * Open H323 Library
 *
 * Copyright (c) 1999-2000 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Open H323 Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions of this code were written with the assisance of funding from 
 * Quicknet Technologies, Inc. http://www.quicknet.net.
 * 
 * Contributor(s): ______________________________________.
 *
 * $Log: ixjwin32.cxx,v $
 * Revision 1.47  2000/07/25 02:07:34  robertj
 * Reduced max range of device numbers as can get some low serial numbers.
 *
 * Revision 1.46  2000/07/14 14:12:06  robertj
 * Added turning off of VAD which results in 1 byte G.723.1 frames that not
 *    everyone supports yet.
 *
 * Revision 1.45  2000/07/12 10:25:07  robertj
 * Added PhoneCARD support on Win9x systems.
 *
 * Revision 1.44  2000/06/20 12:51:23  robertj
 * Changed IXJ driver open so does not change selected line to PSTN.
 *
 * Revision 1.43  2000/06/20 02:22:41  robertj
 * Fixed NT version so can still use serial number to open device.
 *
 * Revision 1.42  2000/06/19 00:31:30  robertj
 * Changed NT device name to be a bit more user friendly.
 *
 * Revision 1.41  2000/06/08 02:33:25  robertj
 * Fixed detection of correct xJack card type under NT.
 * Added ability to use just "0" or "1" instead of "\\.\QTJackDevice0" as device name.
 *
 * Revision 1.40  2000/05/25 02:23:25  robertj
 * Added calls to get volume settings
 *
 * Revision 1.39  2000/05/15 08:38:59  robertj
 * Changed LineJACK PSTN check so is initiated only if state unknown.
 *
 * Revision 1.38  2000/05/02 04:32:27  robertj
 * Fixed copyright notice comment.
 *
 * Revision 1.37  2000/04/30 04:00:33  robertj
 * Changed determination of frame size to use driver ioctl for PCM, uLAw and ALaw.
 *
 * Revision 1.36  2000/04/28 07:00:26  robertj
 * Fixed race condition causing RTP send thread to randomly shut down.
 *
 * Revision 1.35  2000/04/19 01:57:39  robertj
 * Added mixer code to get volume control support on LineJACK;s.
 * Attempt to prevent getting very occassional ReadFrame() failure causing tx channel stop.
 *
 * Revision 1.34  2000/04/12 23:56:37  robertj
 * Fixed detection of PCI PhoneJACK on NT.
 *
 * Revision 1.33  2000/04/06 20:36:25  robertj
 * Fixed some LineJACK compatbility problems (eg DTMF detect stopping).
 *
 * Revision 1.32  2000/04/05 20:55:41  robertj
 * Added caller ID send, and fixed receive for multiple fields.
 *
 * Revision 1.31  2000/04/05 18:04:12  robertj
 * Changed caller ID code for better portability.
 *
 * Revision 1.30  2000/03/30 01:55:13  robertj
 * Added function so silence detection can work on xJack internetl codecs.
 * Fixed length of G.728 codec frames
 *
 * Revision 1.29  2000/03/29 20:59:52  robertj
 * Added function on LID to get available codecs.
 * Improved consistency in "device name".
 * Fixed codec table for G.729 codec
 * Fixed lockup bug with tone/codec interaction.
 *
 * Revision 1.28  2000/03/23 02:48:49  robertj
 * Added calling tone detection code.
 *
 * Revision 1.27  2000/03/17 20:59:42  robertj
 * Fixed line count to be xJACK card dependent.
 * Added support for more xJACK card types.
 *
 * Revision 1.26  2000/02/24 00:35:22  robertj
 * Fixed problem with unresolved SetRemoveDTMF function when not using linux telephony.
 *
 * Revision 1.25  2000/02/16 04:04:37  robertj
 * Fixed bug where IXJ POTS handset never returned off hook.
 *
 * Revision 1.24  2000/01/07 08:28:09  robertj
 * Additions and changes to line interface device base class.
 *
 * Revision 1.23  1999/12/30 09:16:41  robertj
 * Fixed initialisation of driver handle, prevent crash in Close().
 *
 * Revision 1.22  1999/12/23 23:02:36  robertj
 * File reorganision for separating RTP from H.323 and creation of LID for VPB support.
 *
 * Revision 1.21  1999/11/29 04:50:11  robertj
 * Added adaptive threshold calculation to silence detection.
 *
 * Revision 1.20  1999/11/20 04:38:03  robertj
 * Removed potential driver lockups by doing overlapped I/O only on read/write ioctls.
 *
 * Revision 1.19  1999/11/19 09:17:15  robertj
 * Fixed problems with aycnhronous shut down of logical channels.
 *
 * Revision 1.18  1999/11/16 12:43:02  robertj
 * Dixed missing initialise of AEC variable.
 *
 * Revision 1.17  1999/11/16 11:32:06  robertj
 * Added some calling tones.
 *
 * Revision 1.16  1999/11/12 02:25:01  robertj
 * More NT support.
 *
 * Revision 1.15  1999/11/11 23:15:31  robertj
 * Fixed bug where closed driver was not flagged as closed.
 *
 * Revision 1.14  1999/11/11 01:16:57  robertj
 * Added NT support, debounce of phone hook state and wait for line test completion.
 *
 * Revision 1.13  1999/11/06 05:36:19  robertj
 * Fixed problem with read/write locking up when stopping codec.
 *
 * Revision 1.12  1999/11/06 03:32:27  robertj
 * Added volume control functions for record/playback.
 *
 * Revision 1.11  1999/11/05 12:53:40  robertj
 * Fixed warnings on notrace version.
 *
 * Revision 1.10  1999/11/05 10:51:18  robertj
 * Fixed problem with new ixj channel doing incorrect double initialise
 *
 * Revision 1.9  1999/11/05 08:54:41  robertj
 * Rewrite of ixj interface code to fix support for arbitrary codecs.
 *
 * Revision 1.8  1999/11/02 00:24:29  robertj
 * Added GetCallerID() function and implemented some LineJACK code.
 *
 * Revision 1.7  1999/11/01 01:20:26  robertj
 * Added flunction to enabled/disable DTM detection
 *
 * Revision 1.6  1999/11/01 00:47:14  robertj
 * Fixed problems with stopping codecs
 *
 * Revision 1.5  1999/10/30 12:43:25  robertj
 * Fixed "lock up" problem, DTMF problem and added function to get line status.
 *
 * Revision 1.4  1999/10/28 12:21:34  robertj
 * Added AEC support and speakerphone switching button.
 *
 * Revision 1.3  1999/10/27 06:30:31  robertj
 * Added CancelIO command when closing channel.
 *
 * Revision 1.2  1999/10/24 14:51:41  robertj
 * Removed EnableDetectDTMF() as unix ioctl does not exist.
 *
 * Revision 1.1  1999/10/24 12:59:41  robertj
 * Added platform independent support for Quicknet xJACK cards.
 *
 */

#include <ptlib.h>
#include "ixjlid.h"

#include <QTIoctl.h>
#include <ixjDefs.h>


#define new PNEW


static BOOL IsWindows95()
{
  static OSVERSIONINFO version;
  if (version.dwOSVersionInfoSize == 0) {
    version.dwOSVersionInfoSize = sizeof(version);
    GetVersionEx(&version);
  }
  return version.dwPlatformId != VER_PLATFORM_WIN32_NT;
}

#define IsLineJACK() (dwCardType == 3)


/////////////////////////////////////////////////////////////////////////////

OpalIxJDevice::OpalIxJDevice()
{
  hDriver = INVALID_HANDLE_VALUE;
  readStopped = writeStopped = TRUE;
  readFrameSize = writeFrameSize = 480;  // 30 milliseconds of 16 bit PCM data
  aecLevel = AECOff;
  lastDTMFDigit = 0;
}


BOOL OpalIxJDevice::Open(const PString & device)
{
  Close();

  PTRACE(3, "xJack\tOpening IxJ device \"" << device << '"');

  static const char DevicePathPrefix[] = "\\\\.\\QTJackDevice";

  DWORD dwDeviceId = device.AsUnsigned(16);

  PString devicePath;
  if (IsWindows95())
    devicePath = "\\\\.\\Qtipj.vxd";
  else {
    if (dwDeviceId < 100) {
      devicePath = device.Left(device.Find(' '));
      if (strspn(devicePath, "0123456789") == strlen(devicePath))
        devicePath = DevicePathPrefix + devicePath;
    }
    else {
      PStringArray allDevices = GetDeviceNames();
      for (PINDEX dev = 0; dev < allDevices.GetSize(); dev++) {
        PString thisDevice = allDevices[dev];
        if (thisDevice.Find(device) != P_MAX_INDEX) {
          devicePath = thisDevice.Left(thisDevice.Find(' '));
          break;
        }
      }
    }
  }

  hDriver = CreateFile(devicePath,
                       GENERIC_READ | GENERIC_WRITE,
                       FILE_SHARE_WRITE,
                       NULL,
                       OPEN_EXISTING,
                       FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
                       NULL);
  if (hDriver == INVALID_HANDLE_VALUE) {
    osError = ::GetLastError();
    return FALSE;
  }

  if (IsWindows95()) {
    DWORD dwResult = 0;
    if (!IoControl(IOCTL_Device_Open, dwDeviceId, &dwResult) || dwResult == 0) {
      CloseHandle(hDriver);
      hDriver = INVALID_HANDLE_VALUE;
      osError = ENOENT;
      return FALSE;
    }
    dwCardType = dwDeviceId >> 28;
    deviceName = psprintf("%08X", dwDeviceId);
  }
  else {
    PString prefix = devicePath.Mid(sizeof(DevicePathPrefix)-1);
    if (prefix == DevicePathPrefix)
      deviceName = prefix;
    else
      deviceName = devicePath;

    DWORD devId = 0, bytesReturned;
    if (DeviceIoControl(hDriver, IOCTL_Device_GetSerialNumber,
                        NULL, 0, &devId, sizeof(devId), &bytesReturned, NULL) &&
        bytesReturned == sizeof(devId) && devId != 0)
      dwCardType = devId >> 28;
    else {
      PINDEX pos = device.Find('(');
      if (pos != P_MAX_INDEX)
        dwCardType = device[pos+1]-'0';
      else {
        PStringArray names = GetDeviceNames();
        dwCardType = 0;
        for (PINDEX i = 0; i < names.GetSize(); i++) {
          if (names[i].Left(devicePath.GetLength()) == devicePath) {
            pos = names[i].Find('(');
            if (pos != P_MAX_INDEX)
              dwCardType = names[i][pos+1]-'0';
            break;
          }
        }
      }
    }
  }

  IoControl(IOCTL_Idle_Idle);
  IoControl(IOCTL_Codec_SetKHz, 8000);
  IoControl(IOCTL_Idle_SetMasterGain, 15);
  IoControl(IOCTL_Filter_EnableDTMFDetect);

  PTRACE(2, "xJack\tOpened IxJ device " << GetName());
  os_handle = 1;

  return TRUE;
}


BOOL OpalIxJDevice::Close()
{
  if (!IsOpen())
    return FALSE;

  RingLine(0, 0);
  SetLineToLineDirect(0, 1, FALSE);
  IoControl(IOCTL_Record_Stop);
  IoControl(IOCTL_Playback_Stop);

  if (IsWindows95())
    IoControl(IOCTL_Device_Close);

  deviceName = PString();

  BOOL ok = CloseHandle(hDriver);
  hDriver = INVALID_HANDLE_VALUE;
  os_handle = -1;
  return ok;
}


PString OpalIxJDevice::GetName() const
{
  switch (dwCardType) {
    case 0 :
    case 1 :
      return "PhoneJACK (" + deviceName + ')';

    case 3 :
      return "LineJACK (" + deviceName + ')';

    case 4 :
      return "PhoneJACK-Lite (" + deviceName + ')';
  }

  return "xJACK (" + deviceName + ')';
}


unsigned OpalIxJDevice::GetLineCount()
{
  return IsLineJACK() ? NumLines : 1;
}


BOOL OpalIxJDevice::IsLinePresent(unsigned line)
{
  if (line >= GetLineCount())
    return FALSE;

  if (line != PSTNLine)
    return FALSE;

  DWORD dwResult = 0;
  do {
    if (!IoControl(IOCTL_DevCtrl_GetLineTestResult, 0, &dwResult))
      return FALSE;
    if (dwResult == 0xffffffff) {
      IoControl(IOCTL_DevCtrl_LineTest);
      dwResult = LINE_TEST_TESTING;
    }
  } while (dwResult == LINE_TEST_TESTING);

  return dwResult == LINE_TEST_OK;
}


BOOL OpalIxJDevice::IsLineOffHook(unsigned line)
{
  if (line >= GetLineCount())
    return FALSE;

  DWORD dwResult = 0;

  if (line == PSTNLine) {
    DWORD dwResult = 0;
    if (!IoControl(IOCTL_DevCtrl_GetLineOnHook, 0, &dwResult))
      return FALSE;
    return dwResult == 0;
  }

  BOOL state = FALSE;

  for (PINDEX retry = 0; retry < 10; retry++) {
    dwResult = 0;
    if (!IoControl(IsLineJACK() ? IOCTL_DevCtrl_GetLinePhoneOnHook
			        : IOCTL_DevCtrl_GetOnHook, 0, &dwResult))
      return FALSE;

    if (dwResult != 0 && state) {
      state = FALSE;
      retry = 0;
    }
    else if (dwResult == 0 && !state) {
      state = TRUE;
      retry = 0;
    }

    Sleep(10);
  }

  return state;
}


BOOL OpalIxJDevice::SetLineOffHook(unsigned line, BOOL newState)
{
  if (line >= GetLineCount())
    return FALSE;

  if (line != PSTNLine)
    return FALSE;

  return IoControl(IOCTL_DevCtrl_LineSetOnHook, !newState);
}


BOOL OpalIxJDevice::IsLineRinging(unsigned line, DWORD * /*cadence*/)
{
  if (line >= GetLineCount())
    return FALSE;

  if (line != PSTNLine)
    return FALSE;

  if (ringTimeout.IsRunning())
    return TRUE;

  DWORD dwResult = 0;
  if (!IoControl(IOCTL_DevCtrl_LineGetRinging, 0, &dwResult) || dwResult == 0)
    return FALSE;

  ringTimeout = 2500;
  return TRUE;
}


BOOL OpalIxJDevice::RingLine(unsigned line, DWORD cadence)
{
  if (line >= GetLineCount())
    return FALSE;

  if (line != POTSLine)
    return FALSE;

  DWORD dwResult = 0;
  return IoControl(IOCTL_DevCtrl_SetRingPattern, cadence, &dwResult) && dwResult != 0;
}


BOOL OpalIxJDevice::IsLineDisconnected(unsigned line)
{
  if (line >= GetLineCount())
    return FALSE;

  if (line != PSTNLine)
    return FALSE;

  DWORD dwResult = 0;
  return IoControl(IOCTL_DevCtrl_GetLineCallerOnHook, 0, &dwResult) && dwResult != 0;
}


BOOL OpalIxJDevice::SetLineToLineDirect(unsigned line1, unsigned line2, BOOL connect)
{
  if (line1 >= GetLineCount() || line2 >= GetLineCount())
    return FALSE;

  if (line1 == line2)
    return FALSE;

  DWORD dwResult = 0;
  return IoControl(IOCTL_DevCtrl_SetPotsToSlic, connect ? 0 : 1, &dwResult) && dwResult != 0;
}


BOOL OpalIxJDevice::IsLineToLineDirect(unsigned line1, unsigned line2)
{
  if (line1 >= GetLineCount() || line2 >= GetLineCount())
    return FALSE;

  if (line1 == line2)
    return FALSE;

  DWORD dwResult = 0;
  return IoControl(IOCTL_DevCtrl_GetPotsToSlic, 0, &dwResult) && dwResult == 0;
}



static const struct {
  RTP_DataFrame::PayloadTypes rtpType;
  unsigned dspBitMask; // bit0=8020,bit1=8021,bit2=8022
  PINDEX frameSize;
  DWORD recordMode;
  DWORD recordRate;
  DWORD playbackMode;
  DWORD playbackRate;
} CodecInfo[] = {
  { RTP_DataFrame::L16_Mono, 0,   0, RECORD_MODE_16LINEAR,   0,                   PLAYBACK_MODE_16LINEAR,   0                     },
  { RTP_DataFrame::PCMU,     7,   0, RECORD_MODE_ULAW,       0,                   PLAYBACK_MODE_ULAW,       0                     },
  { RTP_DataFrame::PCMA,     7,   0, RECORD_MODE_ALAW,       0,                   PLAYBACK_MODE_ALAW,       0                     },
  { RTP_DataFrame::G728,     2,  20, RECORD_MODE_TRUESPEECH, RECORD_RATE_G728,    PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G728    },
  { RTP_DataFrame::G729,     6,  10, RECORD_MODE_TRUESPEECH, RECORD_RATE_G729,    PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G729    },
  { RTP_DataFrame::G7231,    7,  24, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_63, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 }
};


PIntArray OpalIxJDevice::GetPayloadTypes() const
{
  PIntArray codecs;

  DWORD dwIdCode = 0;
  if (((OpalIxJDevice *)this)->IoControl(IOCTL_DevCtrl_GetIdCode, 0, &dwIdCode)) {
    unsigned dspBit = 1 << (dwIdCode&3);
    PINDEX codec = PARRAYSIZE(CodecInfo);
    while (codec-- > 0) {
      if ((CodecInfo[codec].dspBitMask&dspBit) != 0)
        codecs.SetAt(codecs.GetSize(), CodecInfo[codec].rtpType);
    }
  }

  return codecs;
}


BOOL OpalIxJDevice::SetReadCodec(unsigned, RTP_DataFrame::PayloadTypes rtpType)
{
  PTRACE(3, "xJack\tSetReadCodec(" << rtpType << ')');

  PINDEX codec;
  for (codec = 0; codec < PARRAYSIZE(CodecInfo); codec++)
    if (CodecInfo[codec].rtpType == rtpType)
      break;
  if (codec >= PARRAYSIZE(CodecInfo))
    return FALSE;

  IoControl(IOCTL_Record_Stop);

  if (!IoControl(IOCTL_Codec_SetKHz, 8000))
    return FALSE;

  if (!IoControl(IOCTL_Record_SetBufferChannelLimit, 1))
    return FALSE;

  IoControl(IOCTL_Record_DisableVAD);

  DWORD mode;
  do {
    if (!IoControl(IOCTL_Record_SetRECMODE, CodecInfo[codec].recordMode))
      return FALSE;
    if (!IoControl(IOCTL_Record_GetRECMODE, 0, &mode))
      return FALSE;
#if PTRACING
    if (mode != CodecInfo[codec].recordMode)
      PTRACE(3, "xJack\tSetRECMODE failed (" << mode << " -> "
             << CodecInfo[codec].recordMode << "), retrying");
#endif
  } while (mode != CodecInfo[codec].recordMode);

  DWORD rate;
  do {
    if (!IoControl(IOCTL_Record_SetRate, CodecInfo[codec].recordRate))
      return FALSE;
    if (!IoControl(IOCTL_Record_GetRate, 0, &rate))
      return FALSE;
#if PTRACING
    if (rate != CodecInfo[codec].recordRate)
      PTRACE(3, "xJack\tRecord_SetRate failed (" << rate << " -> "
             << CodecInfo[codec].recordRate << "), retrying");
#endif
  } while (rate != CodecInfo[codec].recordRate);

  readFrameSize = CodecInfo[codec].frameSize;
  if (readFrameSize == 0) {
    DWORD frameWords;
    if (!IoControl(IOCTL_Record_GetFrameSize, 0, &frameWords))
      return FALSE;
    readFrameSize = frameWords*2;
  }

  if (!IoControl(IOCTL_Record_Start))
    return FALSE;

  readStopped = FALSE;

  IoControl(IOCTL_Record_DisableVAD);

  if (aecLevel == AECOff)
    IoControl(IOCTL_Speakerphone_AECOff);
  else {
    IoControl(IOCTL_Speakerphone_SetAEC, aecLevel);
    IoControl(IOCTL_Speakerphone_AECOn);
  }
  return TRUE;
}


BOOL OpalIxJDevice::SetWriteCodec(unsigned, RTP_DataFrame::PayloadTypes rtpType)
{
  PTRACE(3, "xJack\tSetWriteCodec(" << rtpType << ')');

  PINDEX codec;
  for (codec = 0; codec < PARRAYSIZE(CodecInfo); codec++)
    if (CodecInfo[codec].rtpType == rtpType)
      break;
  if (codec >= PARRAYSIZE(CodecInfo))
    return FALSE;

  IoControl(IOCTL_Playback_Stop);

  if (!IoControl(IOCTL_Codec_SetKHz, 8000))
    return FALSE;

  if (!IoControl(IOCTL_Playback_SetBufferChannelLimit, 1))
    return FALSE;

  DWORD mode;
  do {
    if (!IoControl(IOCTL_Playback_SetPLAYMODE, CodecInfo[codec].playbackMode))
      return FALSE;
    if (!IoControl(IOCTL_Playback_GetPLAYMODE, 0, &mode))
      return FALSE;
#if PTRACING
    if (mode != CodecInfo[codec].playbackMode)
      PTRACE(2, "xJack\tSetPLAYMODE failed (" << mode << " -> "
             << CodecInfo[codec].playbackMode << "), retrying");
#endif
  } while (mode != CodecInfo[codec].playbackMode);

  DWORD rate;
  do {
    if (!IoControl(IOCTL_Playback_SetRate, CodecInfo[codec].playbackRate))
      return FALSE;
    if (!IoControl(IOCTL_Playback_GetRate, 0, &rate))
      return FALSE;
#if PTRACING
    if (rate != CodecInfo[codec].playbackRate)
      PTRACE(2, "xJack\tPlayback_SetRate failed (" << rate << " -> "
             << CodecInfo[codec].playbackRate << "), retrying");
#endif
  } while (rate != CodecInfo[codec].playbackRate);

  writeFrameSize = CodecInfo[codec].frameSize;
  if (writeFrameSize == 0) {
    DWORD frameWords;
    if (!IoControl(IOCTL_Playback_GetFrameSize, 0, &frameWords))
      return FALSE;
    writeFrameSize = frameWords*2;
  }

  if (!IoControl(IOCTL_Playback_Start))
    return FALSE;

  writeStopped = FALSE;

  return TRUE;
}


BOOL OpalIxJDevice::StopReadCodec(unsigned)
{
  PTRACE(3, "xJack\tStopping read codec");

  readMutex.Wait();
  readStopped = TRUE;
  IoControl(IOCTL_Record_Stop);
  readMutex.Signal();

  return TRUE;
}


BOOL OpalIxJDevice::StopWriteCodec(unsigned)
{
  PTRACE(3, "xJack\tStopping write codec");

  writeMutex.Wait();
  writeStopped = TRUE;
  IoControl(IOCTL_Playback_Stop);
  writeMutex.Signal();

  return TRUE;
}


PINDEX OpalIxJDevice::GetReadFrameSize(unsigned)
{
  return readFrameSize;
}


PINDEX OpalIxJDevice::GetWriteFrameSize(unsigned)
{
  return writeFrameSize;
}


BOOL OpalIxJDevice::ReadFrame(unsigned, void * buf)
{
  PWaitAndSignal mutex(readMutex);
  if (readStopped)
    return FALSE;

  DWORD dwBytesReturned = 0;
  if (!IoControl(IOCTL_Device_Read, NULL, 0, buf, readFrameSize, &dwBytesReturned, TRUE))
    return FALSE;

  if ((PINDEX)dwBytesReturned == readFrameSize)
    return TRUE;

  osError = ERROR_READ_FAULT;
  return FALSE;
}


BOOL OpalIxJDevice::WriteFrame(unsigned, const void * buf)
{
  PWaitAndSignal mutex(writeMutex);
  if (writeStopped)
    return FALSE;

  DWORD dwResult = 0;
  DWORD dwBytesReturned = 0;
  return IoControl(IOCTL_Device_Write, (void *)buf, writeFrameSize,
                   &dwResult, sizeof(dwResult), &dwBytesReturned, TRUE);
}


unsigned OpalIxJDevice::GetAverageSignalLevel(unsigned)
{
  DWORD dwLevel;
  if (!IoControl(IOCTL_Record_GetAvgRecordLevel, 0, &dwLevel))
    return UINT_MAX;

  return dwLevel;
}


BOOL OpalIxJDevice::EnableAudio(unsigned line, BOOL enable)
{
  if (line >= GetLineCount())
    return FALSE;

  DWORD dwSource = ANALOG_SOURCE_SPEAKERPHONE;

  if (enable)
    dwSource = line == POTSLine ? ANALOG_SOURCE_POTSPHONE : ANALOG_SOURCE_PSTNLINE;

  if (IsLineJACK())
    IoControl(IOCTL_DevCtrl_SetLineJackMode,
                   dwSource == ANALOG_SOURCE_PSTNLINE ? LINEJACK_MODE_LINEJACK
                                                      : LINEJACK_MODE_PHONEJACK);

  return IoControl(IOCTL_DevCtrl_SetAnalogSource, dwSource);
}


BOOL OpalIxJDevice::SetRecordVolume(unsigned, unsigned volume)
{
  if (!IsLineJACK())
    return IoControl(IOCTL_Record_SetVolume, volume);

  MIXER_LINE mixer;
  mixer.dwLineID = 0;

  DWORD dwSize = 0;
  if (!IoControl(IOCTL_Mixer_GetRecordLineControls,
                 &mixer, sizeof(mixer), &mixer, sizeof(mixer), &dwSize, FALSE))
    return FALSE;

  mixer.dwLeftVolume = mixer.dwRightVolume = volume*256;

  DWORD dwReturn;
  return IoControl(IOCTL_Mixer_SetRecordLineControls,
                   &mixer, sizeof(mixer), &dwReturn, sizeof(dwReturn), &dwSize, FALSE);
}


BOOL OpalIxJDevice::SetPlayVolume(unsigned, unsigned volume)
{
  if (!IsLineJACK())
    return IoControl(IOCTL_Playback_SetVolume, volume);

  MIXER_LINE mixer;
  mixer.dwLineID = 0;

  DWORD dwSize = 0;
  if (!IoControl(IOCTL_Mixer_GetPlaybackLineControls,
                 &mixer, sizeof(mixer), &mixer, sizeof(mixer), &dwSize, FALSE))
    return FALSE;

  mixer.dwLeftVolume = mixer.dwRightVolume = volume*256;

  DWORD dwReturn;
  return IoControl(IOCTL_Mixer_SetPlaybackLineControls,
                   &mixer, sizeof(mixer), &dwReturn, sizeof(dwReturn), &dwSize, FALSE);
}


BOOL OpalIxJDevice::GetRecordVolume(unsigned, unsigned & volume)
{
  if (!IsLineJACK()) {
    DWORD value;
    if (!IoControl(IOCTL_Record_GetVolume, 0, &value))
      return FALSE;
    volume = value;
    return TRUE;
  }

  MIXER_LINE mixer;
  mixer.dwLineID = 0;

  DWORD dwSize = 0;
  if (!IoControl(IOCTL_Mixer_GetRecordLineControls,
                 &mixer, sizeof(mixer), &mixer, sizeof(mixer), &dwSize, FALSE))
    return FALSE;

  volume = mixer.dwLeftVolume/256;
  return TRUE;
}


BOOL OpalIxJDevice::GetPlayVolume(unsigned, unsigned & volume)
{
  if (!IsLineJACK()) {
    DWORD value;
    if (!IoControl(IOCTL_Playback_GetVolume, 0, &value))
      return FALSE;
    volume = value;
    return TRUE;
  }

  MIXER_LINE mixer;
  mixer.dwLineID = 0;

  DWORD dwSize = 0;
  if (!IoControl(IOCTL_Mixer_GetPlaybackLineControls,
                 &mixer, sizeof(mixer), &mixer, sizeof(mixer), &dwSize, FALSE))
    return FALSE;

  volume = mixer.dwLeftVolume/256;
  return TRUE;
}


OpalLineInterfaceDevice::AECLevels OpalIxJDevice::GetAEC(unsigned)
{
  return aecLevel;
}


BOOL OpalIxJDevice::SetAEC(unsigned, AECLevels level)
{
  PWaitAndSignal mutexr(readMutex);
  PWaitAndSignal mutexw(writeMutex);

  aecLevel = level;

  // 8022 on xJACK goes mad when you set AEC unless have read/write going.
  if (readStopped && writeStopped)
    return TRUE;

  return IoControl(IOCTL_Speakerphone_SetAEC, level);
}


BOOL OpalIxJDevice::GetCallerID(unsigned, PString & idString, BOOL full)
{
  idString = PString();

  BYTE buffer[256];
  DWORD dwReturn = 0;
  DWORD dwBytesReturned;
  if (!IoControl(IOCTL_DevCtrl_LineGetCallerID,
                 buffer, sizeof(buffer),
                 &dwReturn, sizeof(dwReturn),
                 &dwBytesReturned, FALSE))
    return FALSE;

  PTRACE(3, "IxJ\tCaller ID:\n"
         << hex << setprecision(2)
         << PBYTEArray(buffer, sizeof(buffer))
         << dec);

  if (buffer[0] != 128)
    return FALSE;

  PString name, timeStr;

  PINDEX totalLength = buffer[1];
  PINDEX pos = 2;
  while (pos < totalLength) {
    switch (buffer[pos]) {
      case 1 :
        timeStr = PString((char *)&buffer[pos+2], buffer[pos+1]);
        break;
      case 2 :
        idString = PString((char *)&buffer[pos+2], buffer[pos+1]);
        break;
      case 7 :
        name = PString((char *)&buffer[pos+2], buffer[pos+1]);
        break;
    }
    pos += buffer[pos+1]+2;
  }

  if (full) {
    PTime now;
    PTime theTime(0, timeStr(6,7).AsUnsigned(), timeStr(4,5).AsUnsigned(),
                  timeStr(2,3).AsUnsigned(), timeStr(0,1).AsUnsigned(), now.GetYear());
    idString += '\t' + theTime.AsString(PTime::ShortDateTime) + '\t' + name;
  }

  return TRUE;
}


BOOL OpalIxJDevice::SetCallerID(unsigned line, const PString & idString)
{
  if (line != POTSLine)
    return FALSE;

  if (idString.IsEmpty())
    return TRUE;

  PString name, number;
  PTime theTime;

  PStringArray fields = idString.Tokenise('\t', TRUE);
  switch (fields.GetSize()) {
    case 3 :
      name = fields[2];
    case 2 :
      theTime = PTime(fields[1]);
    case 1 :
     number = fields[0];
     break;
    default :
      return FALSE;
  }

  PINDEX numberLength = number.GetLength();
  PINDEX nameLength = name.GetLength();

  char buffer[256];
  buffer[0] = 1;
  buffer[1] = '\x80';
  buffer[2] = (char)(14+numberLength+nameLength);
  buffer[3] = 1;
  buffer[4] = 8;
  sprintf(&buffer[5],
          "%02u%02u%02u%02u",
          theTime.GetMonth(),
          theTime.GetDay(),
          theTime.GetHour(),
          theTime.GetMinute());
  buffer[13] = 2;
  buffer[14] = (char)numberLength;
  strcpy(&buffer[15], number);
  buffer[15+numberLength] = 7;
  buffer[16+numberLength] = (char)nameLength;
  strcpy(&buffer[17+numberLength], name);

#ifdef IOCTL_FSK_SetMsgData
  DWORD dwReturn = 0;
  DWORD dwBytesReturned;
  return IoControl(IOCTL_FSK_SetMsgData,
                 buffer, 17+numberLength+nameLength,
                 &dwReturn, sizeof(dwReturn),
                 &dwBytesReturned, FALSE);
#else
  return FALSE;
#endif
}


BOOL OpalIxJDevice::PlayDTMF(unsigned line,
                             const char * digits,
                             DWORD onTime, DWORD offTime)
{
  while (*digits != '\0') {
    DWORD dwToneIndex;
    switch (*digits++) {
      case '0' :
        dwToneIndex = IDLE_TONE_0;
        break;
      case '1' :
        dwToneIndex = IDLE_TONE_1;
        break;
      case '2' :
        dwToneIndex = IDLE_TONE_2;
        break;
      case '3' :
        dwToneIndex = IDLE_TONE_3;
        break;
      case '4' :
        dwToneIndex = IDLE_TONE_4;
        break;
      case '5' :
        dwToneIndex = IDLE_TONE_5;
        break;
      case '6' :
        dwToneIndex = IDLE_TONE_6;
        break;
      case '7' :
        dwToneIndex = IDLE_TONE_7;
        break;
      case '8' :
        dwToneIndex = IDLE_TONE_8;
        break;
      case '9' :
        dwToneIndex = IDLE_TONE_9;
        break;
      case '*' :
        dwToneIndex = IDLE_TONE_STAR;
        break;
      case '#' :
        dwToneIndex = IDLE_TONE_POUND;
        break;
      case 'A' :
        dwToneIndex = IDLE_TONE_A;
        break;
      case 'B' :
        dwToneIndex = IDLE_TONE_B;
        break;
      case 'C' :
        dwToneIndex = IDLE_TONE_C;
        break;
      case 'D' :
        dwToneIndex = IDLE_TONE_D;
        break;
      case ',' :
        dwToneIndex = IDLE_TONE_NOTONE;
        Sleep(2000);
        break;
      default :
        dwToneIndex = IDLE_TONE_NOTONE;
        Sleep(onTime+offTime);
        break;
    }

    if (dwToneIndex != IDLE_TONE_NOTONE)
      if (!InternalPlayTone(line, dwToneIndex, onTime, offTime, TRUE))
        return FALSE;
  }

  return TRUE;
}


char OpalIxJDevice::ReadDTMF(unsigned)
{
  DWORD dwNewDigit;
  if (!IoControl(IOCTL_Filter_GetDTMFDigit, lastDTMFDigit, &dwNewDigit))
    return '\0';

  if (dwNewDigit == 0 || dwNewDigit == lastDTMFDigit)
    return '\0';

  lastDTMFDigit = dwNewDigit;

  static char const dtmf[16] = {
    'D','1','2','3','4','5','6','7','8','9','*','0','#','A','B','C'
  };
  PTRACE(3, "IxJ\tDetected DTMF tone: " << dtmf[dwNewDigit&0xf]);

  return dtmf[dwNewDigit&0xf];
}


BOOL OpalIxJDevice::GetRemoveDTMF(unsigned)
{
  return FALSE;
}


BOOL OpalIxJDevice::SetRemoveDTMF(unsigned, BOOL /*state*/)
{
  return FALSE;
}


OpalLineInterfaceDevice::CallProgressTones OpalIxJDevice::IsToneDetected(unsigned line)
{
  if (line >= GetLineCount())
    return NumTones;

  if (!EnableAudio(line, TRUE))
    return NumTones;

  DWORD dwResult;
  if (!IoControl(IOCTL_Filter_LineMonitor, 0, &dwResult))
    return NumTones;

  dwResult &= 0x0f00;

  if (dwResult == 0x300)
    return DialTone;

  if (dwResult == 0x600)
    return RingTone;

  if (dwResult == 0xc00)
    return BusyTone;

#if PTRACING
  if (dwResult != 0)
    PTRACE(1, "xJack\tUnkown tone detected: " << hex << dwResult << dec);
#endif
  return NumTones;
}


BOOL OpalIxJDevice::PlayTone(unsigned line, CallProgressTones tone)
{
  switch (tone) {
    case DialTone :
      return InternalPlayTone(line, IDLE_TONE_DIAL, 0, 0, FALSE);
    case RingTone :
      return InternalPlayTone(line, IDLE_TONE_RING, 0, 0, FALSE);
    case BusyTone :
      return InternalPlayTone(line, IDLE_TONE_BUSY, 0, 0, FALSE);
    default :
      return InternalPlayTone(line, IDLE_TONE_NOTONE, 0, 0, FALSE);
  }
}


BOOL OpalIxJDevice::IsTonePlaying(unsigned)
{
  return PTimer::Tick() < toneSendCompletionTime;
}


BOOL OpalIxJDevice::StopTone(unsigned)
{
  PTRACE(3, "xJack\tStopping tones");

  PWaitAndSignal mutexr(readMutex);
  PWaitAndSignal mutexw(writeMutex);

  if (!IoControl(IOCTL_Idle_StopTone))
    return FALSE;

  // If doing a tone stop, need to restart read and write codec data
  if (!readStopped)
    IoControl(IOCTL_Record_Start);

  if (!writeStopped)
    IoControl(IOCTL_Playback_Start);

  return TRUE;
}


BOOL OpalIxJDevice::InternalPlayTone(unsigned line,
                                     DWORD toneIndex,
                                     DWORD onTime, DWORD offTime,
                                     BOOL synchronous)
{
  StopTone(line);

  PTRACE(3, "xJack\tPlaying tone: "
         << toneIndex << ' ' << onTime << ' ' << offTime << ' ' << synchronous);

  IDLE_TONE tone;

  tone.dwToneIndex = toneIndex;
  tone.dwToneOnPeriod = onTime;
  tone.dwToneOffPeriod = offTime;
  tone.dwDuration = tone.dwToneOnPeriod+tone.dwToneOffPeriod;
  tone.dwMasterGain = 15;

  DWORD dwReturn = 0;
  DWORD dwBytesReturned;
  if (!IoControl(IOCTL_Idle_PlayTone,
                 &tone, sizeof(tone),
                 &dwReturn, sizeof(dwReturn),
                 &dwBytesReturned, FALSE) ||
      dwBytesReturned != sizeof(dwReturn) ||
      dwReturn == 0)
    return FALSE;

  toneSendCompletionTime = PTimer::Tick() + (int)tone.dwDuration - 1;
  if (synchronous)
    Sleep(tone.dwDuration);

  return TRUE;
}


BOOL OpalIxJDevice::SetCountryCode(T35CountryCodes country)
{
  OpalLineInterfaceDevice::SetCountryCode(country);
  return TRUE;
}


PStringArray OpalIxJDevice::GetDeviceNames()
{
  PStringArray array;

  PINDEX i;

  if (IsWindows95()) {
    PStringArray devices = PSoundChannel::GetDeviceNames(PSoundChannel::Player);
    for (i = 0; i < devices.GetSize(); i++) {
      if (devices[i].Find("Internet") != P_MAX_INDEX &&
            (devices[i].Find("JACK") != P_MAX_INDEX ||
             devices[i].Find("PhoneCARD") != P_MAX_INDEX)) {
	PINDEX pos = devices[i].Find('(');
	array[array.GetSize()] = devices[i](pos+1, devices[i].Find(')')-1);
      }
    }
  }
  else {
    const char qtKey[] = "HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services"
                                                   "\\QTJACK\\Parameters\\";
    PString devbase;
    RegistryKey qtJack(qtKey, RegistryKey::ReadOnly);
    for (i = 0; qtJack.EnumKey(i, devbase); i++) {
      if (strncmp(devbase, "Device", 6) == 0) {
        RegistryKey device(qtKey + devbase, RegistryKey::ReadOnly);
        PString pnpdev;
        if (device.QueryValue("PnP Device", pnpdev)) {
          PString devpath = "\\\\.\\QTJack" + devbase;
          HANDLE hDriver = CreateFile(devpath,
                                      GENERIC_READ | GENERIC_WRITE,
                                      FILE_SHARE_WRITE,
                                      NULL,
                                      OPEN_EXISTING,
                                      FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
                                      NULL);
          if (hDriver != INVALID_HANDLE_VALUE) {
            if (strncmp(pnpdev, "ISAPNP", 6) == 0)
              array.SetAt(array.GetSize(), new PCaselessString(devpath + " (" + pnpdev.Mid(pnpdev.FindLast('\\')+1) + ')'));
            else if (strncmp(pnpdev, "PCI", 3) == 0) {
              DWORD devId = 0, bytesReturned;
              if (DeviceIoControl(hDriver, IOCTL_Device_GetSerialNumber,
                                  NULL, 0, &devId, sizeof(devId), &bytesReturned, NULL) &&
                  bytesReturned == sizeof(devId) && devId != 0)
                array.SetAt(array.GetSize(), new PCaselessString(devpath + psprintf(" (%08X)", devId)));
            }
            CloseHandle(hDriver);
          }
        }
      }
    }
  }

  return array;
}


BOOL OpalIxJDevice::IoControl(DWORD dwIoControlCode,
                              DWORD inParam,
                              DWORD * outParam)
{
  DWORD dwDummy;
  if (outParam == NULL)
    outParam = &dwDummy;

  DWORD dwBytesReturned = 0;
  return IoControl(dwIoControlCode, &inParam, sizeof(DWORD),
                   outParam, sizeof(DWORD), &dwBytesReturned, FALSE) &&
         dwBytesReturned == sizeof(DWORD);
}


BOOL OpalIxJDevice::IoControl(DWORD dwIoControlCode,
                               LPVOID lpInBuffer,
                               DWORD nInBufferSize,
                               LPVOID lpOutBuffer,
                               DWORD nOutBufferSize,
                               LPDWORD lpdwBytesReturned,
                               BOOL wait)
{
  if (hDriver == INVALID_HANDLE_VALUE)
    return FALSE;

  OVERLAPPED overlap;
  memset(&overlap, 0, sizeof(overlap));
  if (wait)
    overlap.hEvent = CreateEvent(0, TRUE, 0, NULL);

  DWORD newError = ERROR_SUCCESS;
  if (!DeviceIoControl(hDriver,
                      dwIoControlCode,
                      lpInBuffer,
                      nInBufferSize,
                      lpOutBuffer,
                      nOutBufferSize,
                      lpdwBytesReturned,
                      wait ? &overlap : NULL)) {
    newError = ::GetLastError();
    while (newError == ERROR_IO_PENDING) {
      if (GetOverlappedResult(hDriver, &overlap, lpdwBytesReturned, TRUE))
        newError = ERROR_SUCCESS;
      else
        newError = ::GetLastError();
    }
  }

  if (wait)
    CloseHandle(overlap.hEvent);

  osError = newError;
  return newError == ERROR_SUCCESS;
}



/////////////////////////////////////////////////////////////////////////////
