/*
*########################################################################
*########################################################################
*########################################################################
*                                                               
*   COPYRIGHT (c) 1998, 1999 by TransNexus, LLC                          
*                                                                    
*   This software contains proprietary and confidential information  
*   of TransNexus, LLC. Except as may be set forth in the license    
*   agreement under which this software is supplied, use, disclosure, 
*   or reproduction is prohibited without the prior, express, written
*   consent of TransNexus, LLC.                                      
*                                     
*
*
*
*
*
*
*#########################################################################
*#########################################################################
*#########################################################################
*/

#include <malloc.h>
#include <stdio.h>
#include "aglobal.h"
#include "bsafe.h"

#include "osp.h"
#include "ospcryptowrap.h"
#include "osptnlog.h"
#include "osperrno.h"

#ifdef  OSPC_HWE_CRYPTOSWIFT
#include    "bswift.h"
#endif


static B_ALGORITHM_METHOD *Chooser[] = 
{

#ifdef  OSPC_HWE_CRYPTOSWIFT
    &AM_SwiftDH_KEY_AGREE,
    &AM_SwiftRSA_CRT_DECRYPT,
    &AM_SwiftRSA_CRT_ENCRYPT,
    &AM_SwiftRSA_DECRYPT,
    &AM_SwiftRSA_ENCRYPT,
    &AM_SwiftDSA_SIGN,
    &AM_SwiftDSA_VERIFY,
#endif

    &AM_RSA_CRT_ENCRYPT,
    &AM_RSA_ENCRYPT,
    &AM_RSA_DECRYPT,
    &AM_RSA_CRT_DECRYPT,
    &AM_MD5,
    &AM_SHA,
    (B_ALGORITHM_METHOD *)NULL
};


int 
OSPPCryptoWrapDigest(
        unsigned char   *ospvDigest,
        unsigned        *ospvDigestLength,
        unsigned char   *ospvBERAlgorithm,
        unsigned        ospvBERAlgorithmLength,
        unsigned char   *ospvData,
        unsigned        ospvDataLength,
        unsigned char   ospvFlags)
{
    int errorcode = 0;
    unsigned char digestedData[OSPC_CRYPTO_DIGEST_BUFFER_MAXLENGTH];    
    /* MD5 needs 16, SHA1 needs 20 */
    unsigned int digestedDataLength = 0;
    char errmsg[100];

    ITEM algorithmItem;

    B_ALGORITHM_OBJ digester = (B_ALGORITHM_OBJ) NULL;

    OSPTNLOGDUMP(ospvBERAlgorithm, ospvBERAlgorithmLength, "DIGEST: ospvBERAlgorithm");
    OSPTNLOGDUMP(ospvData, ospvDataLength, "DIGEST: ospvData");

    /* Create an algorithm object */
    errorcode = B_CreateAlgorithmObject(&digester);
    if (errorcode)
    {
        sprintf(errmsg, 
            "B-SAFE error %d occurred in Digest", errorcode);
        errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
        OSPM_DBGERRORLOG(errorcode, errmsg);
    }

    if (errorcode == 0)
    {
        /* Setup the algorithm object */    

        algorithmItem.data = ospvBERAlgorithm;
        algorithmItem.len = ospvBERAlgorithmLength;
        if (ospvFlags & OSPC_CRYPTO_FLAG_USE_BER_ALGORITHM)
        {
            /* Try using MD5 first */
            errorcode = B_SetAlgorithmInfo(digester, AI_MD5_BER, 
                (POINTER) &algorithmItem);
            if (errorcode != OSPC_ERR_NO_ERROR)
            {
                /* Algorithm specified was not MD5, try SHA1 */
                errorcode = B_SetAlgorithmInfo(digester, AI_SHA1_BER, 
                    (POINTER) &algorithmItem);
                if (errorcode)
                {
                    sprintf(errmsg, 
                        "B-SAFE error %d occurred in Digest", errorcode);
                    errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
                    OSPM_DBGERRORLOG(errorcode, errmsg);
                }
            }
        }
        else
        {
            if (ospvFlags & OSPC_CRYPTO_FLAG_USE_SHA1_DIGEST)
            {
                errorcode = B_SetAlgorithmInfo(digester, AI_SHA1, 
                    (POINTER) NULL);
            }
            else
            {
                errorcode = B_SetAlgorithmInfo(digester, AI_MD5, 
                    (POINTER) NULL);
            }
        }
    }


    if (errorcode == 0)
    {
        /* Initialize */    
        errorcode = B_DigestInit(digester, (B_KEY_OBJ) NULL,
            Chooser, (A_SURRENDER_CTX *)NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Digest", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_DigestUpdate(digester, ospvData, ospvDataLength, 
            (A_SURRENDER_CTX *) NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Digest", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_DigestFinal(digester, 
            digestedData, &digestedDataLength, 
            sizeof(digestedData),
            (A_SURRENDER_CTX *) NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Digest", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {

        if (ospvDigest == NULL)
        {
            errorcode = OSPC_ERR_CRYPTO_INVALID_POINTER;
            OSPM_DBGERRORLOG(errorcode, 
                "Error allocating space for message digest");
        }

        if (*ospvDigestLength < digestedDataLength)
        {
            errorcode = OSPC_ERR_CRYPTO_NOT_ENOUGH_SPACE;
            OSPM_DBGERRORLOG(errorcode,
                "Not enough space for digest");
        }

        if (errorcode == 0)
        {
            memcpy(ospvDigest, 
                digestedData, digestedDataLength);

            *ospvDigestLength = digestedDataLength;
            OSPTNLOGDUMP(ospvDigest, *ospvDigestLength, "DIGEST: ospvDigest");
        }
    }

    B_DestroyAlgorithmObject(&digester);

    return errorcode;
}


/* This routine only supports RSA Encryption. It assumes Private Key
encryption, but will try the public key algorithm if the private key
algorithm doesn't match the Algorithm Information supplied */
int 
OSPPCryptoWrapEncrypt(
        unsigned char   *ospvEncryptedData,
        unsigned        *ospvEncryptedDataLength,
        unsigned char   *ospvBERAlgorithm,
        unsigned        ospvBERAlgorithmLength,
        unsigned char   *ospvData,
        unsigned        ospvDataLength,
        unsigned char   *ospvBERSignerKey,
        unsigned        ospvBERSignerKeyLength,
        unsigned char   ospvFlags)
{
    int errorcode = 0;
    unsigned char encryptedData[OSPC_CRYPTO_ENCRYPT_BUFFER_MAXLENGTH];      
    unsigned int encryptedDataLength = 0;
    unsigned int updateLength = 0;
    unsigned int finalLength = 0;
    char errmsg[100];

    ITEM algorithmItem;
    ITEM keyItem;

    B_INFO_TYPE keyType = (B_INFO_TYPE)OSPC_OSNULL;

    B_KEY_OBJ signerKey = (B_KEY_OBJ)OSPC_OSNULL;

    B_ALGORITHM_OBJ rsaEncrypter    = (B_ALGORITHM_OBJ) NULL;
    B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ) NULL;

    OSPTNLOGDUMP(ospvData, ospvDataLength, "ENCRYPT: ospvData");
    OSPTNLOGDUMP(ospvBERSignerKey, ospvBERSignerKeyLength, "ENCRYPT: ospvBERSignerKey");

    /* Create an algorithm object */
    errorcode = B_CreateAlgorithmObject(&rsaEncrypter);
    if (errorcode)
    {
        sprintf(errmsg, 
            "B-SAFE error %d occurred in Encrypt", errorcode);
        errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
        OSPM_DBGERRORLOG(errorcode, errmsg);
    }


    if (errorcode == 0)
    {
        /* Setup the algorithm object */    

        algorithmItem.data = ospvBERAlgorithm;
        algorithmItem.len = ospvBERAlgorithmLength;



        if (ospvFlags & OSPC_CRYPTO_FLAG_USE_BER_ALGORITHM)
        {
            keyType = KI_PKCS_RSAPrivateBER;
            errorcode = B_SetAlgorithmInfo(rsaEncrypter, AI_PKCS_RSAPrivateBER, 
                (POINTER) &algorithmItem);
            if (errorcode != OSPC_ERR_NO_ERROR)
            {
                /* Maybe the algorithm doesn't match BER because BER is for
                Encryption using Public key. */

                /* Maybe this is a public key encryption request */
                OSPM_DBGERRORLOG(0, "RSAPrivate algorithm not specified for "
                    "Encrypt - trying RSAPublic algorithm");

                keyType = KI_RSAPublicBER;
                errorcode = B_SetAlgorithmInfo(rsaEncrypter, 
                    AI_PKCS_RSAPublicBER, 
                    (POINTER) &algorithmItem);
            }

        }
        else
        {
            if (!(ospvFlags & OSPC_CRYPTO_FLAG_ENCRYPTED_WITH_PUBLIC_KEY))
            {
                /* Normally encrypt using private key */
                keyType = KI_PKCS_RSAPrivateBER;
                errorcode = B_SetAlgorithmInfo(rsaEncrypter, 
                    AI_PKCS_RSAPrivate, 
                    (POINTER) NULL);
            }
            else
            {
                /* This is alternative to encrypt for receiver only */
                keyType = KI_RSAPublicBER;
                errorcode = B_SetAlgorithmInfo(rsaEncrypter, 
                    AI_PKCS_RSAPublic, 
                    (POINTER) NULL);
            }
        }

        if (errorcode == 0)
        {
            /* Create a random algorithm object - need one for public
                key encryption. */
            errorcode = B_CreateAlgorithmObject(&randomAlgorithm);
            if (errorcode)
            {
                sprintf(errmsg, 
                    "B-SAFE error %d occurred in Encrypt", errorcode);
                errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
                OSPM_DBGERRORLOG(errorcode, errmsg);
            }
        }   

        if (errorcode == 0)
        {
            /* Set the random algorithm object */
            errorcode = B_SetAlgorithmInfo(randomAlgorithm, AI_MD5Random,
                (POINTER) NULL);
            if (errorcode)
            {
                sprintf(errmsg, 
                    "B-SAFE error %d occurred in Encrypt", errorcode);
                errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
                OSPM_DBGERRORLOG(errorcode, errmsg);
            }
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_CreateKeyObject(&signerKey);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Encrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }

        if (errorcode == 0)
        {
            keyItem.data = ospvBERSignerKey;
            keyItem.len = ospvBERSignerKeyLength;

            errorcode = B_SetKeyInfo(signerKey, keyType, 
                (POINTER) &keyItem); 
            if (errorcode)
            {
                sprintf(errmsg, 
                    "B-SAFE error %d occurred in Encrypt", errorcode);
                errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
                OSPM_DBGERRORLOG(errorcode, errmsg);
            }
        }
    }

    if (errorcode == 0)
    {

        /* Initialize */    
        errorcode = B_EncryptInit(rsaEncrypter, (B_KEY_OBJ) signerKey,
            Chooser, (A_SURRENDER_CTX *)NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Encrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_EncryptUpdate(rsaEncrypter, encryptedData, &updateLength,
            OSPC_CRYPTO_ENCRYPT_BUFFER_MAXLENGTH, 
            ospvData, ospvDataLength, randomAlgorithm, 
            (A_SURRENDER_CTX *) NULL);

        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Encrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_EncryptFinal(rsaEncrypter, 
            encryptedData+updateLength, &finalLength, 
            (OSPC_CRYPTO_ENCRYPT_BUFFER_MAXLENGTH - updateLength),
            randomAlgorithm, (A_SURRENDER_CTX *) NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Encrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {

        encryptedDataLength = updateLength+finalLength;

        if (ospvEncryptedData == NULL)
        {
            errorcode = OSPC_ERR_CRYPTO_INVALID_POINTER;
            OSPM_DBGERRORLOG(errorcode, 
                "Error allocating space for encrypted data");
        }

        if (*ospvEncryptedDataLength < encryptedDataLength)
        {
            errorcode = OSPC_ERR_CRYPTO_NOT_ENOUGH_SPACE;
            OSPM_DBGERRORLOG(errorcode,
                "Not enough space for encrypted data");
        }


        if (errorcode == 0)
        {
            memcpy(ospvEncryptedData, 
                encryptedData, encryptedDataLength);

            *ospvEncryptedDataLength = encryptedDataLength;
            OSPTNLOGDUMP(ospvEncryptedData, *ospvEncryptedDataLength, "ENCRYPT: ospvEncryptedData");
        }
    }

    B_DestroyAlgorithmObject(&rsaEncrypter);
    B_DestroyAlgorithmObject(&randomAlgorithm);
    B_DestroyKeyObject(&signerKey);

    return errorcode;
}

int 
OSPPCryptoWrapDecrypt(
        unsigned char   *ospvDecryptedData,
        unsigned        *ospvDecryptedDataLength,
        unsigned char   *ospvBERAlgorithm,
        unsigned        ospvBERAlgorithmLength,
        unsigned char   *ospvEncryptedData,
        unsigned        ospvEncryptedDataLength,
        unsigned char   *ospvBERReaderKey,
        unsigned        ospvBERReaderKeyLength,
        unsigned char   ospvFlags)
{

    int errorcode = 0;
    unsigned char decryptedData[OSPC_CRYPTO_ENCRYPT_BUFFER_MAXLENGTH];      
    unsigned int decryptedDataLength = 0;
    unsigned int updateLength = 0;
    unsigned int finalLength = 0;
    char errmsg[100];

    ITEM algorithmItem;
    ITEM keyItem;

    B_INFO_TYPE keyType = (B_INFO_TYPE)OSPC_OSNULL;

    B_KEY_OBJ readerKey = (B_KEY_OBJ)OSPC_OSNULL;

    B_ALGORITHM_OBJ rsaDecrypter    = (B_ALGORITHM_OBJ) NULL;
    B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ) NULL;

    OSPM_ARGUSED(ospvFlags);

    OSPTNLOGDUMP(ospvEncryptedData, ospvEncryptedDataLength, "DECRYPT: ospvEncryptedData");
    OSPTNLOGDUMP(ospvBERReaderKey, ospvBERReaderKeyLength, "DECRYPT: ospvBERReaderKey");

    /* Create an algorithm object */
    errorcode = B_CreateAlgorithmObject(&rsaDecrypter);
    if (errorcode)
    {
        sprintf(errmsg, 
            "B-SAFE error %d occurred in Decrypt", errorcode);
        errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
        OSPM_DBGERRORLOG(errorcode, errmsg);
    }


    if (errorcode == 0)
    {
        /* Setup the algorithm object */    

        algorithmItem.data = ospvBERAlgorithm;
        algorithmItem.len = ospvBERAlgorithmLength;

        if (ospvFlags & OSPC_CRYPTO_FLAG_USE_BER_ALGORITHM)
        {
            keyType = KI_RSAPublicBER;
            errorcode = B_SetAlgorithmInfo(rsaDecrypter, AI_PKCS_RSAPublicBER, 
                (POINTER) &algorithmItem);
            if (errorcode != OSPC_ERR_NO_ERROR)
            {
                /* Maybe the algorithm doesn't match BER because BER is for
                Decryption using Public key. */

                /* Maybe this is a public key encryption request */
                OSPM_DBGERRORLOG(0, "RSAPublic algorithm not specified for "
                    "Decrypt - trying RSAPrivate algorithm");

                keyType = KI_PKCS_RSAPrivateBER;
                errorcode = B_SetAlgorithmInfo(rsaDecrypter, 
                    AI_PKCS_RSAPrivateBER, 
                    (POINTER) &algorithmItem);
            }

        }
        else
        {
            if (!(ospvFlags & OSPC_CRYPTO_FLAG_ENCRYPTED_WITH_PUBLIC_KEY))
            {
                /* Normally decrypt using public key */
                keyType = KI_RSAPublicBER;
                errorcode = B_SetAlgorithmInfo(rsaDecrypter, 
                    AI_PKCS_RSAPublic, 
                    (POINTER) NULL);
            }
            else
            {
                /* This is alternative to decrypt for receiver only */
                keyType = KI_PKCS_RSAPrivateBER;
                errorcode = B_SetAlgorithmInfo(rsaDecrypter, 
                    AI_PKCS_RSAPrivate, 
                    (POINTER) NULL);
            }
        }

    }

    if (errorcode == 0)
    {
        errorcode = B_CreateKeyObject(&readerKey);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Decrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }

        if (errorcode == 0)
        {
            keyItem.data = ospvBERReaderKey;
            keyItem.len = ospvBERReaderKeyLength;

            errorcode = B_SetKeyInfo(readerKey, keyType, 
                (POINTER) &keyItem); 
            if (errorcode)
            {
                sprintf(errmsg, 
                    "B-SAFE error %d occurred in Decrypt", errorcode);
                errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
                OSPM_DBGERRORLOG(errorcode, errmsg);
            }
        }
    }

    if (errorcode == 0)
    {

        /* Initialize */    
        errorcode = B_DecryptInit(rsaDecrypter, (B_KEY_OBJ) readerKey,
            Chooser, (A_SURRENDER_CTX *)NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Decrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_DecryptUpdate(rsaDecrypter, decryptedData, &updateLength,
            OSPC_CRYPTO_ENCRYPT_BUFFER_MAXLENGTH, 
            ospvEncryptedData, ospvEncryptedDataLength,
            randomAlgorithm, (A_SURRENDER_CTX *) NULL);

        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Decrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_DecryptFinal(rsaDecrypter, 
            decryptedData+updateLength, &finalLength, 
            (OSPC_CRYPTO_ENCRYPT_BUFFER_MAXLENGTH - updateLength),
            randomAlgorithm, (A_SURRENDER_CTX *) NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Decrypt", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        decryptedDataLength = (updateLength + finalLength);

        if (ospvDecryptedData == NULL)
        {
            errorcode = OSPC_ERR_CRYPTO_UNABLE_TO_ALLOCATE_SPACE;
            OSPM_DBGERRORLOG(errorcode, 
                "Error allocating space for decrypted data");
        }

        if (errorcode == 0)
        {
            memcpy(ospvDecryptedData, 
                decryptedData, decryptedDataLength);

            *ospvDecryptedDataLength = decryptedDataLength;
            OSPTNLOGDUMP(ospvDecryptedData, *ospvDecryptedDataLength, "DECRYPT: ospvDecryptedData");
        }
    }

    B_DestroyAlgorithmObject(&rsaDecrypter);
    B_DestroyAlgorithmObject(&randomAlgorithm);
    B_DestroyKeyObject(&readerKey);

    return errorcode;
}

int 
OSPPCryptoWrapVerify(
        unsigned char   *ospvData,
        unsigned        ospvDataLength,
        unsigned char   *ospvSignature,
        unsigned        ospvSignatureLength,
        unsigned char   *ospvBERReaderKey,
        unsigned        ospvBERReaderKeyLength,
        unsigned char   ospvFlags)
{
    int errorcode = 0;
    char errmsg[100];

    ITEM keyItem;

    B_KEY_OBJ readerKey = (B_KEY_OBJ)OSPC_OSNULL;

    B_ALGORITHM_OBJ rsaVerifier = (B_ALGORITHM_OBJ) NULL;

    OSPM_ARGUSED(ospvFlags);

    OSPTNLOGDUMP(ospvData, ospvDataLength, "VERIFY: ospvData");
    OSPTNLOGDUMP(ospvSignature, ospvSignatureLength, "VERIFY: ospvSignature");
    OSPTNLOGDUMP(ospvBERReaderKey, ospvBERReaderKeyLength, "VERIFY: ospvBERReaderKey");

    /* Create an algorithm object */
    errorcode = B_CreateAlgorithmObject(&rsaVerifier);
    if (errorcode)
    {
        sprintf(errmsg, 
            "B-SAFE error %d occurred in Verify", errorcode);
        errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
        OSPM_DBGERRORLOG(errorcode, errmsg);
    }


    if (errorcode == 0)
    {
        /* Setup the algorithm object */    
        errorcode = B_SetAlgorithmInfo(rsaVerifier, AI_MD5WithRSAEncryption,
            (POINTER) NULL);

        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Verify", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_CreateKeyObject(&readerKey);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Verify", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }

        if (errorcode == 0)
        {
            keyItem.data = ospvBERReaderKey;
            keyItem.len = ospvBERReaderKeyLength;

            errorcode = B_SetKeyInfo(readerKey, KI_RSAPublicBER, 
                (POINTER) &keyItem); 
            if (errorcode)
            {
                sprintf(errmsg, 
                    "B-SAFE error %d occurred in Verify", errorcode);
                errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
                OSPM_DBGERRORLOG(errorcode, errmsg);
            }
        }
    }

    if (errorcode == 0)
    {

        /* Initialize */    
        errorcode = B_VerifyInit(rsaVerifier, (B_KEY_OBJ) readerKey,
            Chooser, (A_SURRENDER_CTX *)NULL);
        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Verify", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_VerifyUpdate(rsaVerifier, ospvData, ospvDataLength, 
            (A_SURRENDER_CTX *) NULL);

        if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Verify", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    if (errorcode == 0)
    {
        errorcode = B_VerifyFinal(rsaVerifier, 
            ospvSignature, ospvSignatureLength, 
            (B_ALGORITHM_OBJ) NULL, 
            (A_SURRENDER_CTX *) NULL);
        if (errorcode == BE_SIGNATURE)
        {
            errorcode = OSPC_ERR_CRYPTO_SIGNATURE_DID_NOT_VERIFY;
        }
        else if (errorcode)
        {
            sprintf(errmsg, 
                "B-SAFE error %d occurred in Verify", errorcode);
            errorcode = OSPC_ERR_CRYPTO_IMPLEMENTATION_SPECIFIC_ERROR;
            OSPM_DBGERRORLOG(errorcode, errmsg);
        }
    }

    B_DestroyAlgorithmObject(&rsaVerifier);
    B_DestroyKeyObject(&readerKey);

    return errorcode;
}


