/*
*########################################################################
*########################################################################
*########################################################################
*                                                               
*   COPYRIGHT (c) 1998, 1999 by TransNexus, LLC                          
*                                                                    
*   This software contains proprietary and confidential information  
*   of TransNexus, LLC. Except as may be set forth in the license    
*   agreement under which this software is supplied, use, disclosure, 
*   or reproduction is prohibited without the prior, express, written
*   consent of TransNexus, LLC.                                      
*                                     
*
*
*
*
*
*
*#########################################################################
*#########################################################################
*#########################################################################
*/

/*
 * ospssl.h - common SSL object and function definitions along with
 *            wrapper prototypes
 */

#ifndef _OSPSSL_H
#define _OSPSSL_H

#include "osp.h"
#include "osphttp.h"
#include "ospsecurity.h"
#include "ospsslsess.h"

#define OSPPSSLSessionHasContext(sslsess) \
    ((sslsess)->Context != OSPC_OSNULL)

#define OSPPSSLSessionGetContext(sslsess) \
    ((sslsess)->Context)

#ifdef __cplusplus
extern "C" 
{
#endif

    /*
     * OSP SDK SSL Entry Point Macros
     */
    /* called only once for each provider */
#define OSPPSSLSessionInit(secptr)     OSPPSSLWrapInit(secptr) 
#define OSPPSSLSessionCleanup(secptr)  OSPPSSLWrapCleanup(secptr) 

    /*
     * OSP SDK SSL Entry Point Functions
     */
    int  OSPPSSLSessionNew(OSPTHTTP *, OSPTSEC *);
    int  OSPPSSLSessionRead(OSPTHTTP *, void *, unsigned int *, char *);
    int  OSPPSSLSessionWrite(OSPTHTTP *, void *, unsigned int *);
    void OSPPSSLSessionDelete(OSPTBOOL, OSPTSSLSESSION **);

    /*
     * SSL Session Member functions
     */
    OSPTSSLSESSION   *OSPPSSLSessionAlloc(void);
    unsigned          OSPPSSLSessionHasSessionId(OSPTSSLSESSION *);

    /* these member functions are called by OSPPSSLSessionNew() */
    int               OSPPSSLSessionInitialize(OSPTHTTP *, OSPTSEC *); 
    int               OSPPSSLSessionNegotiate(OSPTHTTP *);


    /*
     * SSL implementation wrapper functions 
     */
    int             OSPPSSLWrapInit(void *);
    void            OSPPSSLWrapCleanup(void *);
    int             OSPPSSLWrapSessionContextNew(void *, void *);
    int             OSPPSSLWrapSessionContextDelete(OSPTSSLSESSION *);
    int             OSPPSSLWrapAttachConnection(OSPTSSLSESSION *, void *);
    int             OSPPSSLWrapSessionGracefulShutdown(OSPTSSLSESSION *);
    int             OSPPSSLWrapHandshake(OSPTSSLSESSION *);
    int             OSPPSSLWrapGetData(void *, unsigned int *, 
                        OSPTSSLSESSION *);
    int             OSPPSSLWrapSendData(void *, unsigned int *, 
                        OSPTSSLSESSION *);
    int             OSPPSSLWrapGetServerRootCACert(void **,
                        int *, OSPTSSLSESSION *);
    void            OSPPSSLWrapFreeServerRootCACert(void **);

#ifdef __cplusplus
}
#endif

#endif
