/*
*########################################################################
*########################################################################
*########################################################################
*                                                               
*   COPYRIGHT (c) 1998, 1999 by TransNexus, LLC                          
*                                                                    
*   This software contains proprietary and confidential information  
*   of TransNexus, LLC. Except as may be set forth in the license    
*   agreement under which this software is supplied, use, disclosure, 
*   or reproduction is prohibited without the prior, express, written
*   consent of TransNexus, LLC.                                      
*                                     
*
*
*
*
*
*
*#########################################################################
*#########################################################################
*#########################################################################
*/

/*
 * ospsslref.c - SSL API bridge to SSLREF v3.0
 */

#include "osp.h"
#include "ospsocket.h"
#include "ospssl.h"
#include "osputils.h"
#include "ospcomm.h"
#include "ospsecurity.h"

/* SSLREF headers */
#include "ssl.h"
#include "asn1util.h"

#ifdef BSAFE
#include "bsafe.h"
#endif

#define SSLREF_DEFAULTVERSION    SSL_Version_3_0
#define OSPC_MAX_CERT_BUFFER     4096

/*
 * local function prototypes
 */
SSLErr OSPPSSLWrapParsePrivateKey(SSLBuffer, SSLRSAPrivateKey *);
SSLErr OSPPSSLWrapAddCertificates(OSPTSEC *, SSLContext *);
SSLErr OSPPSSLWrapAddDistinguishedName(OSPTSEC *, SSLContext *);

/*
 * internal callback prototypes 
 */
SSLErr SSLREF_AddSessionId(SSLBuffer, SSLBuffer, void *);
SSLErr SSLREF_Alloc(SSLBuffer *, void *);
SSLErr SSLREF_ClientRead(SSLBuffer, uint32 *, void *); 
SSLErr SSLREF_ClientWrite(SSLBuffer, uint32 *, void *); 
SSLErr SSLREF_ConvertTime(uint32 *, void *);
SSLErr SSLREF_DeleteSessionId(SSLBuffer, void *);
SSLErr SSLREF_Free(SSLBuffer *, void *);
SSLErr SSLREF_GetSessionId(SSLBuffer, SSLBuffer *, void *);
SSLErr SSLREF_Random(SSLBuffer, void *);
SSLErr SSLREF_Realloc(SSLBuffer *, uint32, void *);
SSLErr SSLREF_SeedRandom(void **);
SSLErr SSLREF_Time(uint32 *, void *);

int
OSPPSSLWrapInit(void *ospvRef)
{
    int errorcode = OSPC_ERR_NO_ERROR;
    OSPM_DBGENTER(("ENTER: OSPPSSLWrapInit()\n"));
    OSPM_ARGUSED(ospvRef);
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapInit()\n"));
    return errorcode;
}

void
OSPPSSLWrapCleanup(void *ospvRef)
{
    OSPM_DBGENTER(("ENTER: OSPPSSLWrapCleanup()\n"));
    OSPM_ARGUSED(ospvRef);
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapCleanup()\n"));
    return;
}

int
OSPPSSLWrapSessionContextNew(
    void *ospvConnection,
    void *ospvContextRef)
{
    int                 errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr              sslref_errcode = SSLNoErr;
    SSLContext        **ctx            = OSPC_OSNULL;
    SSLProtocolVersion  version        = SSLREF_DEFAULTVERSION;
    OSPTSSLSESSION     *sslsession     = OSPC_OSNULL;
    OSPTSEC            *security       = OSPC_OSNULL;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapSessionContextNew()\n"));

    sslsession = ((OSPTHTTP *)ospvConnection)->SSLSession;
    security   = (OSPTSEC *)ospvContextRef;

    ctx = (SSLContext **)&sslsession->Context;
    OSPM_MALLOC(*ctx, SSLContext, SSLContextSize()); 
    if (*ctx == OSPC_OSNULL)
    {
        errorcode = OSPC_ERR_SSL_MALLOC_FAILED;
        OSPM_DBGERRORLOG(errorcode, "malloc of new SSL Context failed");
    }
    else
    {
        /*
         * configure the callback functions for the context 
         */
        if ((sslref_errcode = SSLInitContext(*ctx)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, "SSLInitContext() failed");
        }

        else if ((sslref_errcode = SSLSetAllocFunc(*ctx, SSLREF_Alloc)) 
            != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetAllocFunc() failed");
        }

        else if ((sslref_errcode = SSLSetFreeFunc(*ctx, SSLREF_Free)) 
            != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetFreeFunc() failed");
        }

        else if ((sslref_errcode = SSLSetReallocFunc(*ctx, 
            SSLREF_Realloc)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetReallocFunc() failed");
        }

        else if ((sslref_errcode = SSLSetRandomFunc(*ctx, SSLREF_Random)) 
            != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetRandomFunc() failed");
        }

        else if ((sslref_errcode = SSLSetTimeFunc(*ctx, SSLREF_Time)) 
            != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetTimeFunc() failed");
        }

        else if ((sslref_errcode = SSLSetConvertTimeFunc(*ctx, 
            SSLREF_ConvertTime)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetConvertTimeFunc() failed");
        }

        else if ((sslref_errcode = SSLSetReadFunc(*ctx, 
            SSLREF_ClientRead)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetReadFunc() failed");
        }

        else if ((sslref_errcode = SSLSetWriteFunc(*ctx, 
            SSLREF_ClientWrite)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetWriteFunc() failed");
        }

        else if ((sslref_errcode = SSLSetAddSessionFunc(*ctx, 
            SSLREF_AddSessionId)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetAddSessionFunc() failed");
        }

        else if ((sslref_errcode = SSLSetGetSessionFunc(*ctx, 
            SSLREF_GetSessionId)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetGetSessionFunc() failed");
        }
        else if ((sslref_errcode = SSLSetDeleteSessionFunc(*ctx, 
            SSLREF_DeleteSessionId)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetDeleteSessionFunc() failed");
        }

#ifdef BSAFE

        else if ((sslref_errcode = SSLREF_SeedRandom(
            &(sslsession->RandomRef))) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLREF_SeedRandom() failed");
        }

        SSLSetRandomRef(*ctx, 
            *(B_ALGORITHM_OBJ*)(sslsession->RandomRef));
#endif

        /*
         * set protocol version
         */
        if (errorcode == OSPC_ERR_NO_ERROR &&
            (sslref_errcode = SSLSetProtocolVersion(*ctx, version)) 
            != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_VERSION_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetProtocolVersion() failed");
        }

        /*
         * set session reference
         */
        if (errorcode == OSPC_ERR_NO_ERROR &&
            (sslref_errcode = SSLSetSessionRef(*ctx, 
            ospvConnection)) != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_SESSION_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, "SSLSetSessionRef() failed");
        }

        /*
         * set  protocol side (client)
         */
        if (errorcode == OSPC_ERR_NO_ERROR &&
            (sslref_errcode = SSLSetProtocolSide(*ctx, SSL_ClientSide)) 
            != SSLNoErr)
        {
            errorcode = OSPC_ERR_SSL_INIT_CTX_FAILED;
            OSPM_DBGERRORLOG(sslref_errcode, 
                "SSLSetProtocolSide() failed");
        }

        /*
         * client authentication requirements
         */
#ifdef OSPC_ENABLE_SSL_CLIENT_AUTHENTICATION
        if (errorcode == OSPC_ERR_NO_ERROR)
        {
            SSLBuffer key;
            SSLRSAPrivateKey rsaKey;

            errorcode = OSPPSecGetPrivateKeyData(security, &(key.data), (unsigned int *)&(key.length));
            if (errorcode == OSPC_ERR_NO_ERROR)
            {
                sslref_errcode = OSPPSSLWrapParsePrivateKey(key, &rsaKey);
                if (sslref_errcode == SSLNoErr)
                {
                    sslref_errcode = SSLSetPrivateKey(*ctx, &rsaKey);
                    if (sslref_errcode == SSLNoErr)
                    { 
                        sslref_errcode = OSPPSSLWrapAddCertificates(security, *ctx);
                        if (sslref_errcode == SSLNoErr)
                        {
                            sslref_errcode = OSPPSSLWrapAddDistinguishedName(security, *ctx);
                            if (sslref_errcode != SSLNoErr)
                            {
                                errorcode = OSPC_ERR_SSL_ADD_DN_FAILED;
                                OSPM_DBGERRORLOG(sslref_errcode, 
                                    "OSPPSSLWrapAddDistinguishedName() failed");
                            }
                        } 
                        else
                        {
                            errorcode = OSPC_ERR_SSL_ADD_CERTS_FAILED;
                            OSPM_DBGERRORLOG(sslref_errcode, 
                                "OSPPSSLWrapAddCertificates() failed");
                        }
                    }
                    else
                    {
                        errorcode = OSPC_ERR_SSL_SET_PRIVKEY_FAILED;
                        OSPM_DBGERRORLOG(sslref_errcode, 
                            "SSLSetPrivateKey() failed");
                    }
                }
                else
                {
                    OSPM_DBGERRORLOG(sslref_errcode, 
                        "OSPPSSLWrapParsePrivateKey() failed");
                    errorcode = OSPC_ERR_SSL_PARSE_PRIVKEY_FAILED;
                }
            }
            else
            {
                OSPM_DBGERRORLOG(errorcode, 
                    "OSPPSecGetPrivateKeyData() failed");
            }
        }
#endif /* OSPC_ENABLE_SSL_CLIENT_AUTHENTICATION */
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapSessionContextNew() (%d)\n", errorcode));
    return errorcode;
}

SSLErr
    OSPPSSLWrapAddDistinguishedName(
    OSPTSEC    *ospvSecurity,
    SSLContext *ospvSSLContext)
{
    OSPM_ARGUSED(ospvSSLContext);
    OSPM_ARGUSED(ospvSecurity);
    return SSLNoErr; 
}

SSLErr
    OSPPSSLWrapAddCertificates(
    OSPTSEC    *ospvSecurity,
    SSLContext *ospvSSLContext)
{

    SSLErr sslref_errcode = SSLNoErr;
    int    errorcode      = OSPC_ERR_NO_ERROR,
           certindex      = -1,
           previndex      = -1,
           certorder      = 0;
    unsigned int certlen  = 0;
    SSLBuffer cert;

    unsigned char certbuf[OSPC_MAX_CERT_BUFFER];

    while (errorcode == OSPC_ERR_NO_ERROR && sslref_errcode == SSLNoErr)
    {
        certlen = OSPC_MAX_CERT_BUFFER;
        previndex = certindex;
        cert.length = 0;
        errorcode = OSPPSecValidCertChain(ospvSecurity, &certindex, certbuf, &certlen);
        if (errorcode == OSPC_ERR_NO_ERROR)
        {
            if (certlen > 0)
            {
                OSPM_MALLOC(cert.data, unsigned char, certlen+1);
                if (cert.data != OSPC_OSNULL)
                {
                    cert.length = certlen;
                    OSPM_MEMCPY(cert.data, certbuf, certlen);
                    /* add cert */
                    sslref_errcode = SSLAddCertificate(ospvSSLContext, cert, certorder++, 0);
                    if (sslref_errcode == SSLNoErr)
                    {
                        if (previndex == certindex && previndex >= 0)
                        {
                            /* verify cert chain only  - if valid, break */
                            cert.data = 0;
                            cert.length = 0;
                            sslref_errcode = SSLAddCertificate(ospvSSLContext, cert, 0, 1);
                            if (sslref_errcode == SSLNoErr)
                            {
                                break;
                            }
                            else
                            {
                                OSPM_DBGERRORLOG(sslref_errcode, "SSLAddCertificate() verify failed");
                            }
                        }
                        else
                        {
                            if (certorder == 1) /* local cert */
                            {
                                OSPTASN1OBJECT      *localcert  = OSPC_OSNULL;
                                OSPTASN1ELEMENTINFO *eInfo      = OSPC_OSNULL;
                                SSLBuffer            dn;

                                errorcode = OSPPX509CertCreate(cert.data, &localcert);
                                if (errorcode == OSPC_ERR_NO_ERROR)
                                {
                                   /* Get the local certificate subject name */
                                   errorcode = OSPPASN1ObjectGetElementByDataRef(localcert,
                                               &eInfo, OSPEDRID_CERT_SUBJECT);
                               
                                   if (errorcode == OSPC_ERR_NO_ERROR)
                                   {
                                       errorcode = OSPPASN1ElementGetElementData(eInfo, (unsigned char **)&(dn.data), 
                                          (unsigned int *)&(dn.length));
                                       if (errorcode == OSPC_ERR_NO_ERROR)
                                       {
                                          sslref_errcode = SSLAddDistinguishedName(ospvSSLContext, dn);
                                          if (sslref_errcode != SSLNoErr)
                                          {
                                              OSPM_DBGERRORLOG(sslref_errcode, "SSLAddDistinguishedName() failed");
                                          }
                                       }
                                       else
                                       {
                                          OSPM_DBGERRORLOG(errorcode, "OSPPASN1ElementGetElementData() failed");
                                       }
                                   }
                                   else
                                   {
                                      OSPM_DBGERRORLOG(errorcode, "OSPPASN1ObjectGetElementByDataRef() failed");
                                   }
                                }
                            }

                        }
                    }
                    else
                    {
                        OSPM_DBGERRORLOG(sslref_errcode, "SSLAddCertificate() failed");
                    }
                }
                else
                {
                    sslref_errcode = SSLMemoryErr;
                    OSPM_DBGERRORLOG(sslref_errcode, "OSPPSecValidCertChain() malloc failed");
                }
            }
            else
            {
                sslref_errcode = SSLUnknownErr;
                OSPM_DBGERRORLOG(sslref_errcode, "OSPPSecValidCertChain() invalid cert length");
            }
        }
        else
        {
            OSPM_DBGERRORLOG(errorcode, "OSPPSecValidCertChain() failed");
            sslref_errcode = SSLUnknownErr;
        }
    } 
    return sslref_errcode;
}

SSLErr
    OSPPSSLWrapParsePrivateKey(
    SSLBuffer         privateKey, 
    SSLRSAPrivateKey *key)
{
    SSLErr          err;
    ASN1Type        rsaKeyOuter, rsaKey[9];
    int             count;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapParsePrivateKey()\n"));

    count = 1;
    if ((err = ASNParseBER(privateKey, &rsaKeyOuter, &count)) != 0 || count != 1)
    {
        OSPM_DBGERRORLOG(err, "count 1: OSPPSSLWrapParsePrivateKey() failed");
        return err;
    }
    count = 9;
    if ((err = ASNParseBER(rsaKeyOuter.contents, rsaKey, &count)) != 0 || count != 9)
    {
        OSPM_DBGERRORLOG(err, "count 9: OSPPSSLWrapParsePrivateKey() failed");
        return err;
    }
    {
        A_PKCS_RSA_PRIVATE_KEY  privKey;
        int                     rsaErr;

#define COPY_SSLBUFFER_TO_ITEM(b,i) i.data = b.data; i.len = b.length;

        COPY_SSLBUFFER_TO_ITEM(rsaKey[1].contents, privKey.modulus);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[2].contents, privKey.publicExponent);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[3].contents, privKey.privateExponent);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[4].contents, privKey.prime[0]);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[5].contents, privKey.prime[1]);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[6].contents, privKey.primeExponent[0]);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[7].contents, privKey.primeExponent[1]);
        COPY_SSLBUFFER_TO_ITEM(rsaKey[8].contents, privKey.coefficient);

        if ((rsaErr = B_CreateKeyObject(key)) != 0)
        {
            OSPM_DBGERRORLOG(rsaErr, "OSPPSSLWrapParsePrivateKey() B_CreateKeyObject failed");
            return SSLUnknownErr;
        }
        if ((rsaErr = B_SetKeyInfo(*key, KI_PKCS_RSAPrivate, (POINTER)&privKey)) != 0)
        {
            OSPM_DBGERRORLOG(rsaErr, "OSPPSSLWrapParsePrivateKey() B_SetKeyInfo failed");
            return SSLUnknownErr;
        }
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapSessionContextNew()\n"));
    return SSLNoErr;
}  /* OSPPSSLWrapParsePrivateKey */


int
OSPPSSLWrapAttachConnection(
    OSPTSSLSESSION *ospvSSLSession, 
    void           *ospvConnection)
{
    int         errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr      sslref_errcode = SSLNoErr;
    SSLContext *ctx            = OSPC_OSNULL;
    SSLBuffer   sslbuffer      = { (uint32)1, (uint8 *)"" };

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapAttachConnection()\n"));
    ctx = (SSLContext *)OSPPSSLSessionGetContext(ospvSSLSession);

    /*
     * set  protocol side (client)
     */
    if ((sslref_errcode = SSLSetIORef(ctx, ospvConnection)) != SSLNoErr)
    {
        errorcode = OSPC_ERR_SSL_ATTACH_SOCK_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLSetIORef() failed");
    }

    /*
     * this function is required to indicate to the SSL library that
     * sessions can be reused.
     */
    if ((sslref_errcode = SSLSetPeerID(ctx, sslbuffer)) != SSLNoErr)
    {
        errorcode = OSPC_ERR_SSL_ATTACH_SOCK_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLSetPeerId() failed");
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapAttachConnection() (%d)\n", errorcode));
    return errorcode;
}

int
OSPPSSLWrapHandshake( 
    OSPTSSLSESSION *ospvSSLSession)
{
    int    errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapHandshake()\n"));
    /*
     * Do SSL handshake with server
     */
    do
    {
        sslref_errcode = SSLHandshake(
            (SSLContext *)OSPPSSLSessionGetContext(ospvSSLSession));
    } while (sslref_errcode == SSLWouldBlockErr);

    if (sslref_errcode != SSLNoErr)
    {
        errorcode = OSPC_ERR_SSL_HANDSHAKE_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLHandshake() failed");
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapHandshake() (%d)\n", errorcode));
    return errorcode;
}

int
OSPPSSLWrapSessionGracefulShutdown(
    OSPTSSLSESSION *ospvSSLSession)
{
    int    errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapSessionGracefulShutdown()\n"));
    OSPM_DBGSEC(("SSL  : OSPPSSLWrapSessionGracefulShutdown() SSLClose() called\n"));

    if ((sslref_errcode = SSLClose(
        (SSLContext *)OSPPSSLSessionGetContext(ospvSSLSession))) 
        != SSLNoErr)
    {
        errorcode = OSPC_ERR_SSL_CLOSE_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLClose() failed");
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapSessionGracefulShutdown() (%d)\n", errorcode));
    return errorcode;
}

int
OSPPSSLWrapSessionContextDelete(
    OSPTSSLSESSION *ospvSSLSession)
{
    int    errorcode      = OSPC_ERR_NO_ERROR;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapSessionContextDelete()\n"));

#ifdef BSAFE
    B_DestroyAlgorithmObject(
        (B_ALGORITHM_OBJ *)ospvSSLSession->RandomRef);
#endif

    OSPM_DBGSEC(
        ("SSL  : OSPPSSLWrapSessionContextDelete() SSLDeleteContext() called (%d)\n",
        errorcode));

    SSLDeleteContext((SSLContext *)
        OSPPSSLSessionGetContext(ospvSSLSession));

    OSPM_FREE(ospvSSLSession->Context);
    ospvSSLSession->Context = (void *)OSPC_OSNULL;

    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapSessionContextDelete() (%d)\n", errorcode));
    return errorcode;
}

int
OSPPSSLWrapGetData(
    void           *ospvBuffer, 
    unsigned int   *ospvLength, 
    OSPTSSLSESSION *ospvSSLSession)
{
    int               errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr            sslref_errcode = SSLNoErr;
    SSLContext        *ctx           = OSPC_OSNULL;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapGetData()\n"));
    ctx = (SSLContext *)OSPPSSLSessionGetContext(ospvSSLSession);

    do
    {
        sslref_errcode = SSLRead(ospvBuffer, (uint32 *)ospvLength, ctx);
    } while (sslref_errcode == SSLWouldBlockErr);

    if (sslref_errcode != SSLNoErr)
    {
        errorcode = OSPC_ERR_SSL_READ_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLRead() failed");
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapGetData() (%d)\n", errorcode));
    return errorcode;
}

int
OSPPSSLWrapSendData(
    void           *ospvBuffer, 
    unsigned int   *ospvLength, 
    OSPTSSLSESSION *ospvSSLSession)
{
    int         errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr      sslref_errcode = SSLNoErr;
    SSLContext *ctx            = OSPC_OSNULL;

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapSendData()\n"));
    ctx = (SSLContext *)OSPPSSLSessionGetContext(ospvSSLSession);

    do
    {
        sslref_errcode = SSLWrite(ospvBuffer, (uint32 *)ospvLength, ctx);
    } while (sslref_errcode == SSLWouldBlockErr);

    if (sslref_errcode != SSLNoErr)
    {
        errorcode = OSPC_ERR_SSL_WRITE_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLWrite() failed");
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapSendData() (%d)\n", errorcode));
    return errorcode;
}

int
OSPPSSLWrapGetServerRootCACert(
    void           **ospvRootCACert,
    int             *ospvRootCACertLen,
    OSPTSSLSESSION  *ospvSSLSession)
{
    int         errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr      sslref_errcode = SSLNoErr;
    SSLContext *ctx            = OSPC_OSNULL;
    SSLBuffer   certBuf        = { 0, OSPC_OSNULL };

    OSPM_DBGENTER(("ENTER: OSPPSSLWrapGetServerRootCACert()\n"));
    ctx = (SSLContext *)OSPPSSLSessionGetContext(ospvSSLSession);

    /*
     * now get the top level cert - Mr. CA Root
     */
    sslref_errcode = SSLGetPeerCertificate(ctx, 0, &certBuf);
    if (sslref_errcode == SSLNoErr)
    {
        *ospvRootCACert    = certBuf.data;
        *ospvRootCACertLen = certBuf.length;
    } 
    else
    {
        errorcode = OSPC_ERR_SSL_GETCERT_FAILED;
        OSPM_DBGERRORLOG(sslref_errcode, "SSLGetPeerCertificate() failed");
    }
    OSPM_DBGEXIT(("EXIT : OSPPSSLWrapGetServerRootCACert() (%d)\n", errorcode));
    return errorcode;
}

void
OSPPSSLWrapFreeServerRootCACert(
    void **ospvRootCACert)
{
    if (ospvRootCACert && *ospvRootCACert != OSPC_OSNULL)
    {
        OSPM_FREE(*ospvRootCACert);
        *ospvRootCACert = OSPC_OSNULL;
    }
    return;
}


/* ------------------------------------------------------------*/
/* SSLREF-specific Callback Functions                          */
/* ------------------------------------------------------------*/

SSLErr
    SSLREF_Alloc(
    SSLBuffer *ospvBuffer, 
    void      *ospvAllocRef)
{
    SSLErr sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: SSLREF_Alloc()\n"));
    OSPM_ARGUSED(ospvAllocRef);
    OSPM_MALLOC(ospvBuffer->data, unsigned char, ospvBuffer->length);
    if (ospvBuffer->data == OSPC_OSNULL)
    {
        sslref_errcode = SSLMemoryErr;
    }
    else
    {
        OSPM_MEMSET(ospvBuffer->data, 0, ospvBuffer->length);
    }

    OSPM_DBGEXIT(("EXIT : SSLREF_Alloc() (%lx)\n", 
        (unsigned long)ospvBuffer->data));
    return sslref_errcode;
}

SSLErr
    SSLREF_Free(
    SSLBuffer *ospvBuffer, 
    void      *ospvAllocRef)
{
    SSLErr sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: SSLREF_Free() (%lx)\n",
        (unsigned long)ospvBuffer->data));

    OSPM_ARGUSED(ospvAllocRef);

    if (ospvBuffer != OSPC_OSNULL)
    {
        if (ospvBuffer->data != OSPC_OSNULL)
        {
            OSPM_FREE(ospvBuffer->data);
            ospvBuffer->data = OSPC_OSNULL;
        }
        ospvBuffer->length = 0;
    }
    OSPM_DBGEXIT(("EXIT : SSLREF_Free() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_Realloc(
    SSLBuffer *ospvBuffer, 
    uint32     ospvNewSize, 
    void      *ospvAllocRef)
{  
    unsigned char *newData        = OSPC_OSNULL;
    SSLErr         sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: SSLREF_Realloc()\n"));

    OSPM_ARGUSED(ospvAllocRef);
    OSPM_REALLOC(newData, ospvBuffer->data, unsigned char, ospvNewSize);
    if (newData == OSPC_OSNULL)
    {
        sslref_errcode = SSLMemoryErr;
        OSPM_FREE(ospvBuffer->data);
        ospvBuffer->data=OSPC_OSNULL;
        ospvBuffer->length=0;
    }
    else
    {
        ospvBuffer->data=newData;
        ospvBuffer->length=ospvNewSize;
    }
    OSPM_DBGEXIT(("EXIT : SSLREF_Realloc() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_Random(
    SSLBuffer ospvBuffer, 
    void     *ospvRandomRef)
{   
    int    result         = 0;
    SSLErr sslref_errcode = SSLNoErr;

#if RSAREF
    R_RANDOM_STRUCT     *rsaRandom = (R_RANDOM_STRUCT*)ospvRandomRef;

    OSPM_DBGENTER(("ENTER: SSLREF_Random()\n"));
    if (ospvRandomRef == 0)
        sslref_errcode = SSLUnknownErr;

    if ((result = R_GenerateBytes(ospvBuffer.data, ospvBuffer.length, 
        rsaRandom)) != 0)
        sslref_errcode = SSLUnknownErr;

#elif BSAFE
    B_ALGORITHM_OBJ random = (B_ALGORITHM_OBJ)ospvRandomRef;

    OSPM_DBGENTER(("ENTER: SSLREF_Random()\n"));
    if ((result = B_GenerateRandomBytes(random, ospvBuffer.data, 
        ospvBuffer.length, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
        sslref_errcode = SSLUnknownErr;
#endif /* BSAFE / RSAREF */

    OSPM_DBGEXIT(("EXIT : SSLREF_Random() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

/* Seed the random number generator with the time. This is an
 *  extremely bad way to seed a secure random number generator
 *  and basically sacrifices all security, but suffices for
 *  an implementation example. In other words, CHANGE THIS!
 */
SSLErr
    SSLREF_SeedRandom(void **ospvRandomRef)
{  
    time_t t              = 0;
    int    rsaResult      = 0;
    SSLErr sslref_errcode = SSLNoErr;

#if RSAREF
    unsigned int            bytesNeeded;
    static R_RANDOM_STRUCT  rsaRandom;

    OSPM_DBGENTER(("ENTER: SSLREF_SeedRandom()\n"));
    *ospvRandomRef = 0;
    t = time(0);

    if (R_RandomInit(&rsaRandom) != 0)
    {
        sslref_errcode = SSLUnknownErr;
    }
    else
    {
        if (R_GetRandomBytesNeeded(&bytesNeeded, &rsaRandom) != 0)
        {
            sslref_errcode = SSLUnknownErr;
        }
        else
        { 
            while (bytesNeeded > 0)
            {   
                if ((rsaResult = R_RandomUpdate(&rsaRandom, (unsigned char*)&t, 
                    sizeof(time_t))) != 0)
                {
                    sslref_errcode = SSLUnknownErr;
                    break;
                }

                if (bytesNeeded >= sizeof(time_t))
                    bytesNeeded -= sizeof(time_t);
                else
                    bytesNeeded = 0;
            }

            *ospvRandomRef = &rsaRandom;
        }
    }
#elif BSAFE
    static B_ALGORITHM_OBJ  random;
    B_ALGORITHM_METHOD      *chooser[] = { &AM_MD5_RANDOM, 0 };

    OSPM_DBGENTER(("ENTER: SSLREF_SeedRandom()\n"));
    if ((rsaResult = B_CreateAlgorithmObject(&random)) != 0)
    {
        sslref_errcode = SSLUnknownErr;
    }
    else
    {
        if ((rsaResult = B_SetAlgorithmInfo(random, AI_MD5Random, 0)) != 0)
        {
            sslref_errcode = SSLUnknownErr;
        }
        else
        {
            if ((rsaResult = B_RandomInit(random, chooser, NO_SURR)) != 0)
            {
                sslref_errcode = SSLUnknownErr;
            }
            else
            { 
                t = time(0);

                if ((rsaResult = B_RandomUpdate(random, (unsigned char*)&t, sizeof(t), 
                    NO_SURR)) != 0)
                {
                    sslref_errcode = SSLUnknownErr;
                }
                else
                    *ospvRandomRef = &random;
            }
        }
    }
#endif /* BSAFE / RSAREF */

    OSPM_DBGEXIT(("EXIT : SSLREF_SeedRandom() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_Time( 
    uint32 *ospvTimeResult, 
    void   *ospvTimeRef)
{   
    OSPTTIME t;

    OSPM_DBGENTER(("ENTER: SSLREF_Time()\n"));
    OSPM_ARGUSED(ospvTimeRef);
    t = time(0);
    *ospvTimeResult = (uint32)t;
    OSPM_DBGEXIT(("EXIT : SSLREF_Time()\n"));
    return SSLNoErr;
}

SSLErr
    SSLREF_ConvertTime(
    uint32 *ospvTime, 
    void   *ospvTimeRef)
{
    OSPM_DBGENTER(("ENTER: SSLREF_ConvertTime()\n"));
    *ospvTime -= 20000;
    OSPM_ARGUSED(ospvTimeRef);
    OSPM_DBGEXIT(("EXIT : SSLREF_ConvertTime()\n"));
    return SSLNoErr;
}

SSLErr
    SSLREF_ClientRead(
    SSLBuffer ospvBuffer, 
    uint32   *ospvProcessed, 
    void     *ospvConnection)
{
    int      errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr   sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: SSLREF_ClientRead()\n"));
    errorcode = OSPPSockRead((OSPTHTTP *)ospvConnection, ospvBuffer.data, 
        (unsigned int *)&ospvBuffer.length);

    *ospvProcessed = ospvBuffer.length;

    if (errorcode != OSPC_ERR_NO_ERROR)
        sslref_errcode = SSLIOErr;

    OSPM_DBGEXIT(("EXIT : SSLREF_ClientRead() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_ClientWrite(
    SSLBuffer ospvBuffer, 
    uint32   *ospvProcessed, 
    void     *ospvConnection)
{
    int      errorcode      = OSPC_ERR_NO_ERROR;
    SSLErr   sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: SSLREF_ClientWrite()\n"));
    errorcode = OSPPSockWrite((OSPTHTTP *)ospvConnection, ospvBuffer.data, 
        (unsigned int *)&ospvBuffer.length);

    *ospvProcessed = ospvBuffer.length;

    if (errorcode != OSPC_ERR_NO_ERROR)
        sslref_errcode = SSLIOErr;

    OSPM_DBGEXIT(("EXIT : SSLREF_ClientWrite() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_AddSessionId(
    SSLBuffer  ospvKey, 
    SSLBuffer  ospvData, 
    void      *ospvSessionRef)
{
    OSPTSEC           *security       = OSPC_OSNULL;
    OSPTSVCPT         *servicepoint   = OSPC_OSNULL;
    SSLErr             sslref_errcode = SSLNoErr;
    int                errorcode      = OSPC_ERR_NO_ERROR;

    OSPM_DBGENTER(("ENTER: SSLREF_AddSessionId()\n"));
    OSPM_ARGUSED(ospvKey);

    if (ospvSessionRef == OSPC_OSNULL)
    {
        sslref_errcode = SSLMemoryErr;
    }
    else
    {
        servicepoint = ((OSPTHTTP *)ospvSessionRef)->ServicePoint;
        security = OSPPHttpGetSecurity((OSPTHTTP *)ospvSessionRef);

        if (servicepoint == OSPC_OSNULL || security == OSPC_OSNULL)
        {
            sslref_errcode = SSLMemoryErr;
        }
        else
        {
            errorcode = OSPPSecSSLSessionIdNew(security, ospvData.length, ospvData.data,
                servicepoint->HostName, servicepoint->Port);
            if (errorcode != OSPC_ERR_NO_ERROR)
            {
                sslref_errcode = SSLMemoryErr;
            }
        }
    }
    OSPM_DBGEXIT(("EXIT : SSLREF_AddSessionId() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_GetSessionId(
    SSLBuffer  ospvKey, 
    SSLBuffer *ospvData, 
    void      *ospvSessionRef)
{
    OSPTSEC        *security       = OSPC_OSNULL;
    OSPTSVCPT      *servicepoint   = OSPC_OSNULL;
    SSLErr          sslref_errcode = SSLNoErr;
    int             errorcode      = OSPC_ERR_NO_ERROR;

    OSPM_DBGENTER(("ENTER: SSLREF_GetSessionId()\n"));
    OSPM_ARGUSED(ospvKey);

    if (ospvSessionRef == OSPC_OSNULL)
    {
        sslref_errcode = SSLMemoryErr;
    }
    else
    {
        servicepoint = ((OSPTHTTP *)ospvSessionRef)->ServicePoint;
        security = OSPPHttpGetSecurity((OSPTHTTP *)ospvSessionRef);

        if (servicepoint == OSPC_OSNULL || security == OSPC_OSNULL)
        {
            sslref_errcode = SSLMemoryErr;
        }
        else
        {
            errorcode = OSPPSecSSLSessionIdGet(security, &(ospvData->length), 
                &(ospvData->data), servicepoint->HostName, servicepoint->Port);
            if (errorcode != OSPC_ERR_NO_ERROR)
            {
                sslref_errcode = SSLSessionNotFoundErr;
            }
        }
    }
    OSPM_DBGEXIT(("EXIT : SSLREF_GetSessionId() (%d)\n", sslref_errcode));
    return sslref_errcode;
}

SSLErr
    SSLREF_DeleteSessionId(
    SSLBuffer  ospvKey, 
    void      *ospvSessionRef)
{    
    OSPTSEC          *security       = OSPC_OSNULL;
    OSPTSSLSESSION   *session        = OSPC_OSNULL;
    SSLErr            sslref_errcode = SSLNoErr;

    OSPM_DBGENTER(("ENTER: SSLREF_DeleteSessionId()\n"));
    OSPM_ARGUSED(ospvKey);

    if (ospvSessionRef == OSPC_OSNULL)
    {
        sslref_errcode = SSLMemoryErr;
    }
    else
    {
        security = OSPPHttpGetSecurity((OSPTHTTP *)ospvSessionRef);
        session  = ((OSPTHTTP *)ospvSessionRef)->SSLSession;

        if (security != OSPC_OSNULL && OSPPSSLSessionHasSessionId(session))
        {
            OSPPSecSSLSessionIdDelete(security, &(session->SessionId), OSPC_TRUE);
        } 
        else
        {
            sslref_errcode = SSLSessionNotFoundErr;
        }
    }
    OSPM_DBGEXIT(("EXIT : SSLREF_DeleteSessionId() (%d)\n", sslref_errcode));
    return sslref_errcode;
}
