/*
###########################################################################
###########################################################################
###########################################################################
###                                                                     ###
###   COPYRIGHT (c) 1999 by TransNexus, LLC                             ###
###                                                                     ###
###   This software contains proprietary and confidential information   ###
###   of TransNexus, LLC. Except as may be set forth in the licencse    ###
###   agreement under which this software is supplied, use, disclosure, ###
###   or reproduction is prohibited without the prior, express, written ###
###   consent of TransNexus, LLC.                                       ###
###                                                                     ###
###########################################################################
###########################################################################
###########################################################################
*/

/*
 * tnids.h - structures and prototypes for functions that manage
 *           TransNexus transaction IDs
 */

#ifndef tnids_h
#define tnids_h

#include "osp.h"


/*MACROS
#define tnListNewest (tnSentinel.tidOlderPtr)
#define tnListOldest (tnSentinel.tidNewerPtr)
#define tnListEnd    (&tnSentinel)
*/

typedef struct _OSPTTRANSID                  /* info about a transaction ID */
{
    OSPTUINT64     ospmTransactionId;           /* the actual transaction ID value */
    unsigned long  ospmTransactionIdExpires;    /* expiration time */
    struct _OSPTTRANSID    *ospmTransactionIdLessPtr;    /* lower valued ID in tree */
    struct _OSPTTRANSID    *ospmTransactionIdMorePtr;    /* greater valued ID in tree */
    struct _OSPTTRANSID   **ospmTransactionIdParent;     /* parent pointer (for removes) */
    struct _OSPTTRANSID    *ospmTransactionIdOlderPtr;   /* older in list */
    struct _OSPTTRANSID    *ospmTransactionIdNewerPtr;   /* more recent in list */
}OSPTTRANSID;

#include "ospprovider.h"

#ifdef __cplusplus
extern "C" 
{
#endif

    /* Function Definitions */
	OSPTBOOL        OSPPTransIdAdd(OSPTTRANSID *, struct _OSPTPROVIDER *);
	OSPTBOOL        OSPPTransIdCheckAndAdd(OSPTUINT64, unsigned long, struct _OSPTPROVIDER   *);
	void            OSPPTransIdDelete(OSPTTRANSID *);
	void            OSPPTransIdInit(struct _OSPTPROVIDER *);
	OSPTTRANSID    *OSPPTransIdNew(OSPTUINT64, unsigned long);
	void            OSPPTransIdPurge(OSPTTRANSID *, struct _OSPTPROVIDER *);
	void            OSPPTransIdRemove(OSPTTRANSID *, struct _OSPTPROVIDER *);
	unsigned long   OSPPTransIdSecNow(void);
	void            OSPPTransIdTimeAdd(OSPTTRANSID *, struct _OSPTPROVIDER *);
    void            OSPPTransIDTreeDelete(struct _OSPTPROVIDER *);

#ifdef TN_TRANSDBG
	void tnPrintTree(OSPTTRANSID *pTrans);
	void tnTransById(struct _OSPTPROVIDER *);    /* print by transaction ID */
	void tnTransByTime(struct _OSPTPROVIDER *);  /* print by expiration time */
#endif

#ifdef __cplusplus
}
#endif

#endif
