/*
*########################################################################
*########################################################################
*########################################################################
*                                                               
*   COPYRIGHT (c) 1998, 1999 by TransNexus, LLC                          
*                                                                    
*   This software contains proprietary and confidential information  
*   of TransNexus, LLC. Except as may be set forth in the license    
*   agreement under which this software is supplied, use, disclosure, 
*   or reproduction is prohibited without the prior, express, written
*   consent of TransNexus, LLC.                                      
*                                     
*
*
*
*
*
*
*#########################################################################
*#########################################################################
*#########################################################################
*/


/*
 * ospxmlattr.h - Structures and prototypes for generic XML attributes.
 */

#ifndef _OSPXMLATTR_H
#define _OSPXMLATTR_H

#include "osp.h"
#include "osplist.h"

/*
 * An XMLAttr structure represents an XML attribute in its
 * natural form ("natural" in the sense of easiest to reference
 * in code). It includes the attribute's name, as well as the
 * character value. The structure is defined as a linked list
 * so that multiple attributes (e.g. for the same element) can
 * be linked together.
 */

typedef struct OSPSXMLAttr
{
    OSPTLISTLINK ospmXMLAttrLink;
    const char  *ospmXMLAttrName;
    const char  *ospmXMLAttrValue;
}
OSPTXMLATTR;


#define OSPC_MAX_ID_LEN 25

/**/
/*-----------------------------------------------------------------------*
 * macros that emulate functions
 *-----------------------------------------------------------------------*/

/*
 * Note: all macros are also implemented as functions in ospxmlattr.c.
 * For implementation details, see the comments in that file. To replace
 * a macro with a true function, simply comment out the macro definition
 * below.
 */
#ifndef OSPC_DEBUG
#define OSPPXMLAttrGetName(ospvXMLAttr)  (ospvXMLAttr)?((ospvXMLAttr)->ospmXMLAttrName):OSPC_OSNULL
#define OSPPXMLAttrGetValue(ospvXMLAttr) (ospvXMLAttr)?((ospvXMLAttr)->ospmXMLAttrValue):OSPC_OSNULL
#endif
/**/
/*-----------------------------------------------------------------------*
 * function prototypes
 *-----------------------------------------------------------------------*/

OSPTXMLATTR *OSPPXMLAttrNew(const unsigned char *, const unsigned char *);
void         OSPPXMLAttrDelete(OSPTXMLATTR **);

#ifndef OSPPXMLAttrGetName
const char *OSPPXMLAttrGetName(OSPTXMLATTR *);
#endif

#ifndef OSPPXMLAttrGetValue
const char *OSPPXMLAttrGetValue(OSPTXMLATTR *);
#endif

#endif

