/*
 * Program:	Kerberos 5 check password
 *
 * Author:	Mark Crispin
 *		Networks and Distributed Computing
 *		Computing & Communications
 *		University of Washington
 *		Administration Building, AG-44
 *		Seattle, WA  98195
 *		Internet: MRC@CAC.Washington.EDU
 *
 * Date:	1 August 1988
 * Last Edited:	6 October 2000
 *
 * Copyright 2000 by the University of Washington
 *
 *  Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies and that both the
 * above copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the University of Washington not be
 * used in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  This software is made available
 * "as is", and
 * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
 * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Check password
 * Accepts: login passwd struct
 *	    password string
 *	    argument count
 *	    argument vector
 * Returns: passwd struct if password validated, NIL otherwise
 */

struct passwd *checkpw (struct passwd *pw,char *pass,int argc,char *argv[])
{
  krb5_context ctx;
  krb5_timestamp now;
  krb5_principal client,server;
  krb5_creds crd;
  struct passwd *ret = NIL;
  if (*pass) {			/* only if password non-empty */
    krb5_init_context (&ctx);	/* get a context context */
    krb5_init_ets (ctx);
				/* get time, client and server principals */
    if (!krb5_timeofday (ctx,&now) &&
	!krb5_parse_name (ctx,pw->pw_name,&client) &&
	!krb5_build_principal_ext (ctx,&server,
				   krb5_princ_realm (ctx,client)->length,
				   krb5_princ_realm (ctx,client)->data,
				   KRB5_TGS_NAME_SIZE,KRB5_TGS_NAME,
				   krb5_princ_realm (ctx,client)->length,
				   krb5_princ_realm (ctx,client)->data,0)) {
				/* initialize credentials */
      memset (&crd,0,sizeof (krb5_creds));
      crd.client = client;	/* set up client and server credentials */
      crd.server = server;
				/* expire in 3 minutes */
      crd.times.endtime = now + (3 * 60);
      if (!krb5_get_in_tkt_with_password (ctx,NIL,NIL,NIL,NIL,pass,0,&crd,0))
	ret = pw;
				/* don't need server principal any more */
      krb5_free_principal (ctx,server);
    }
    krb5_free_context (ctx);	/* don't need context any more */
  }
  return ret;
}
