
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const rtpCodec_cxx_Version =
    "$Id: rtpCodec.cxx,v 1.10 2000/12/18 23:44:48 bko Exp $";



#include "rtpCodec.h"

#include "cpLog.h"
#include "g711.h"

#include "rtpTypes.h"
#include "RtpPacket.hxx"


RtpPacket*
convertRtpPacketCodec (RtpPayloadType toType, RtpPacket* packet)
{
    RtpPacket* p = new RtpPacket(packet, 1012);
    assert (p);

    int result_size = convertCodec (packet->getPayloadType(), toType,
                                    packet->getPayloadLoc(), p->getPayloadLoc(),
                                    packet->getPayloadUsage());

    p->setPayloadType(toType);
    p->setPayloadUsage(result_size);

    return p;
}


// return value is size of converted data stored at out_data
int
convertCodec(RtpPayloadType fromType, RtpPayloadType toType,
             char* in_data, char* out_data, int len)
{
    int result_len = -1;

    unsigned char* charBuff = (unsigned char*)out_data;
    short* shortBuff = (short*)out_data;
    unsigned char* charPayload = (unsigned char*)in_data;
    short* shortPayload = (short*)in_data;


    switch (fromType)
    {
        case rtpPayloadPCMU :
        {
            switch (toType)
            {
                case rtpPayloadL16_mono :
                case rtpPayloadL16_8k_mono :
                for (int i = 0; i < len; i++)
                    shortBuff[i] = ulaw2linear (charPayload[i]);
                result_len = len * 2;
                break;
                default :
                cpLog(LOG_DEBUG, "Unsupported converstion %d -> %d",
                      fromType, toType);
                break;
            }
        }
        break;
        case rtpPayloadL16_mono :
        case rtpPayloadL16_8k_mono :
        {
            switch (toType)
            {
                case rtpPayloadPCMU :
                for (int i = 0; i < len / 2; i++)
                    charBuff[i] = linear2ulaw (shortPayload[i]);
                result_len = len / 2;
                break;
                default :
                cpLog(LOG_DEBUG, "Unsupported converstion %d -> %d",
                      fromType, toType);
                break;
            }
        }
        break;
        default :
        {
            cpLog(LOG_DEBUG, "Unsupported converstion %d -> %d",
                  fromType, toType);
            break;
        }
    }

    return result_len;
}


void rtp_htonl (RtpPacket* p)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned char tdata;
    if (p->getPayloadType() == rtpPayloadL16_mono)
    {
        unsigned char* data = (unsigned char*) p->getPayloadLoc();
        for (int i = 0; i < p->getPayloadUsage(); i = i + 2)
        {
            tdata = data[i];
            data[i] = data[i + 1];
            data[i + 1] = tdata;
        }
    }
#endif
}


void rtp_ntohl (RtpPacket* p)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned char tdata;
    if (p->getPayloadType() == rtpPayloadL16_mono)
    {
        unsigned char* data = (unsigned char*) p->getPayloadLoc();
        for (int i = 0; i < p->getPayloadUsage(); i = i + 2)
        {
            tdata = data[i];
            data[i] = data[i + 1];
            data[i + 1] = tdata;
        }
    }
#endif
}


void createSilenceCodec (char* buffer, RtpPayloadType type, int len)
{
    switch (type)
    {
        case rtpPayloadPCMU:
        memset (buffer, 0xFF, len);
        break;
        case rtpPayloadL16_mono:   // debug
        case rtpPayloadG729:   // debug
        default:
        cpLog (LOG_ERR, "Can't make exact silence for type: &d", (int)type);
        memset (buffer, 0, len);
        break;
    }
}


// sets the exact networkFormat information, esp network_perSampleSize
void setNetworkFormatCodec()
{
    return ;
}
