/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const Verify_cxx_Version = 
    "$Id: Verify.cxx,v 1.2 2001/02/01 01:35:02 bko Exp $";

#include "Verify.hxx"

TestStatistics* TestStatistics::myInstance = 0;
VMutex TestStatistics::myMutex;

TestStatistics::TestStatistics()
    : myPassed(0),
      myFailed(0)
{
}

TestStatistics::~TestStatistics()
{
}

void
TestStatistics::passed()
{
    Lock lock(myMutex);
    if(!myInstance)
    {
	myInstance = new TestStatistics();
    }
    myInstance->myPassed++;
}


void
TestStatistics::failed()
{
    Lock lock (myMutex);
    if(!myInstance)
    {
	myInstance = new TestStatistics();
    }
    myInstance->myFailed++;
}


int
TestStatistics::finish(const char* filename)
{
    bool anyFailed = false;
    Lock lock (myMutex);
    if(myInstance)
    {
	if(filename)
	{
	    cerr << filename << ": ";
	}
	cerr << "Passed: " << myInstance->myPassed << 
	    "\tFailed: " << myInstance->myFailed << endl;
	if(myInstance->myFailed > 0)
	{
	    anyFailed = true;
	}
	delete myInstance;
    }
    if(anyFailed)
    {
	return -1;
    }
    else
    {
	return 0;
    }
}

#if 0
/** this class exists so that TestStatistics::finish() can be called
 * when the program finishes.  To do this, it calls it in the
 * destructor, and a global instance of the class is instantiated.
 */
class TestStatisticDestructorActivation
{
    public:
	TestStatisticDestructorActivation()
	{
	}

	~TestStatisticDestructorActivation()
	{
	    TestStatistics::finish(0);
	}
};

/// here is the global instance
static TestStatisticDestructorActivation junk;
#endif
